/*
 * Hanashizume no Matsuri for Palm
 * Copyright (c) 2001 TF <tf@denpa.org>
 * All rights reserverd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: cutter.c,v 1.7 2001/04/07 07:37:24 tf Exp $
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

extern unsigned char *get_jpeg(FILE *, long, int *, int *);
extern unsigned char *clip_image(int, int, int, int, unsigned char *);
extern unsigned char *shrink_gray(int, int, int, unsigned char *);
extern void output_ppm(int, int, unsigned char *, const char *);
extern unsigned char *make_palm_bmp(unsigned char *);
extern int lz77_compress(unsigned char *, int, unsigned char *);

#include "resinfo.h"

#undef JPG_OUTPUT_MODE 1

#define TYPE_NORMAL 0x0
#define TYPE_DIFF   0x1

#define NAME_LEN 16
char filename[NAME_LEN];

void
put2b(int val, FILE *fp)
{
  unsigned char b1, b2;

  b1 = val & 0xff;
  b2 = (val >> 8) & 0xff;

  fputc(b2, fp);
  fputc(b1, fp);
}

void
output_file(unsigned char *data, int len, const char *filename, int type, int arg)
{
  FILE *fp;

  fp = fopen(filename, "wb");
  if (fp == NULL) {
    perror("fopen");
    exit(1);
  }

  fputc(type & 0xff, fp);
  if (type == TYPE_DIFF) {
    put2b(arg, fp);
  }
  
  fwrite(data, 1, len, fp);

  fclose(fp);
}

#ifdef JPG_OUTPUT_MODE
void
output_jpeg(FILE *fp, long offset, const char *resname)
{
  int c, p;
  char jpgfile[NAME_LEN];
  FILE *out;
  
  fseek(fp, offset, SEEK_SET);

#ifdef _GNU_SOURCE
  snprintf(jpgfile, NAME_LEN, "%s.jpg", resname);
#else
  sprintf(jpgfile, "%s.jpg", resname);
#endif
  out = fopen(jpgfile, "wb");
  if (out == NULL) {
    perror("fopen");
    exit(1);
  }
  
  p = 0;
  while(1) {
    c = fgetc(fp);
    fputc(c, out);
    if (c == 0xd9 && p == 0xff) {
      break;
    }
    p = c;
  }
  
  fclose(out);
}
#endif

unsigned char *
get_palm_bitmap(FILE *fp, RESINFO *res_list, const char *resname)
{
  RESINFO *r;
  int w, h;
  unsigned char *img1, *img2, *img3, *img4;

  r = search_rlist(res_list, resname);
  /* fprintf(stderr, "%s: %08x\n", resname, (int)r->offset); */
  
#ifdef JPG_MODE
  output_jpeg(fp, r->offset, resname);
  return NULL;
#endif

  img1 = get_jpeg(fp, r->offset, &w, &h);
  if (img1 == NULL) {
    return NULL;
  }
  img2 = clip_image(w, h, 480, 480, img1);
  free(img1);
  
  img3 = shrink_gray(480, 480, 3, img2);
  free(img2);
  
  img4 = make_palm_bmp(img3);

  free(img3);

  return img4;
}

void
calc_xor(unsigned char *dst, unsigned char *from, unsigned char *to)
{
  int i;

  for (i = 0; i < 80 * 160; i++) {
    dst[i] = from[i] ^ to[i];
  }
}

void
parse_cg_filename(const char *name, int *id)
{
  char *str, *p;

  str = strdup(name);

  if (!strncmp(name, "CGHANA", 6)) {
    *id = 2499;
    return;
  } else if (!strncmp(name, "CGBLACK", 7)) {
    *id = 2498;
    return;
  } else if (!strncmp(name, "HNTI01", 6)) {
    *id = 2497;
    return;
  } else if (!strncmp(name, "HNTI02", 6)) {
    *id = 2496;
    return;
  } else if (!strncmp(name, "HNTI03", 6)) {
    *id = 2495;
    return;
  } else if (!strncmp(name, "HNTI04", 6)) {
    *id = 2494;
    return;
  } else if (!strncmp(name, "HNTI05", 6)) {
    *id = 2493;
    return;
  }
  
  for (p = str; *p != '0'; p++);

  *id = strtol(p, NULL, 10);

  free(str);
}

int main(int argc, char *argv[])
{
  FILE *fp, *mfp;
  RESINFO *res_list;
  int i = 0;
  unsigned char *src, *dst, *frm, *to;
  int size;
  char buf[BUFSIZ], resname[NAME_LEN], arg[NAME_LEN];
  int type, diff;
#ifdef DEBUG
  int diff_org_size, diff_is_used;
  unsigned char *dummy_buf;
#endif

  if (argc != 3) {
    fprintf(stderr, "usage: hanacg.dll maptable\n");
    exit(1);
  }
  if ((fp = fopen(argv[1], "rb")) == NULL) {
    fprintf(stderr, "Can't open %s\n", argv[1]);
    exit(1);
  }
  if ((mfp = fopen(argv[2], "r")) == NULL) {
    fprintf(stderr, "Can't open %s\n", argv[2]);
    exit(1);
  }

  res_list = create_rlist();
  scan_dll(fp, res_list);
  
  while (fgets(buf, BUFSIZ, mfp) != NULL) {
#ifdef DEBUG
    diff_is_used = 0;
#endif
    i++;
    if (sscanf(buf, "%s%s", resname, arg) != 2) {
      fprintf(stderr, "input format error at line %d.\n", i);
      exit(1);
    }

    src = get_palm_bitmap(fp, res_list, resname);
    if (src == NULL) {
      continue;
    }

    if (!strcmp(arg, "none")) {
      type = TYPE_NORMAL;
      diff = 0;
      fprintf(stderr, "CG: %s -> ", resname);
#ifdef _GNU_SOURCE
      snprintf(filename, NAME_LEN, "%s.bm", resname);
#else
      sprintf(filename, "%s.bm", resname);
#endif
      fprintf(stderr, "%s\n", filename);
    } else {
      type = TYPE_DIFF;
      parse_cg_filename(arg, &diff);
      fprintf(stderr, "CG: %s:%s -> ", resname, arg);
#ifdef _GNU_SOURCE
      snprintf(filename, NAME_LEN, "%s.bm2", resname);
#else
      sprintf(filename, "%s.bm2", resname);
#endif
      fprintf(stderr, "%s\n", filename);

      frm = src;
      to = get_palm_bitmap(fp, res_list, arg);
      if (to == NULL) {
	fprintf(stderr, "argument is incorrect(line %d).\n", i);
	exit(1);
      }
#ifdef DEBUG
      diff_is_used = 1;
      dummy_buf = (unsigned char *)calloc(80 * 160, sizeof(unsigned char));
      diff_org_size = lz77_compress(frm, 80 * 160, dummy_buf);
      free(dummy_buf);
#endif
      src = (unsigned char *)calloc(80 * 160, sizeof(unsigned char));
      calc_xor(src, frm, to);
      free(to);
    }
    dst = (unsigned char *)calloc(80 * 160, sizeof(unsigned char));
    size = lz77_compress(src, 80 * 160, dst);
#ifdef DEBUG
    if (diff_is_used) {
      if (size > diff_org_size) {
	fprintf(stderr, "Warning: XORed image is larger than simple compression: %s\n", filename);
      }
    }
#endif
    output_file(dst, size, filename, type, diff);
    free(src);
    free(dst);
    src = dst = NULL;
  }
  fclose(mfp);
  fclose(fp);

  return 0;
}
