/*
 * Hanashizume no Matsuri for Palm
 * Copyright (c) 2001 TF <tf@denpa.org>
 * All rights reserverd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: image.c,v 1.1 2001/02/17 05:50:41 tf Exp $
 */
#include <stdio.h>
#include <stdlib.h>

unsigned char *clip_image(int w1, int h1, int w2, int h2, unsigned char *src)
{
  int xmargin, ymargin, x1, y1, x2, y2;
  unsigned char *dst;

  xmargin = (w1 - w2) / 2;
  ymargin = (h1 - h2) / 2;

  dst = (unsigned char *)calloc(w2 * h2 * 3, sizeof(unsigned char));
  if (dst == NULL) {
    perror("calloc");
    exit(1);
  }
  
  for (y2 = 0, y1 = ymargin; y2 < h2; y1++, y2++) {
    for (x2 = 0, x1 = xmargin; x2 < w2; x1++, x2++) {
      dst[(y2 * w2 + x2) * 3] = src[(y1 * w1 + x1) * 3];
      dst[(y2 * w2 + x2) * 3 + 1] = src[(y1 * w1 + x1) * 3 + 1];
      dst[(y2 * w2 + x2) * 3 + 2] = src[(y1 * w1 + x1) * 3 + 2];
    }
  }
  return dst;
}

unsigned char *shrink_gray(int w0, int h0, int factor, unsigned char *src)
{
  int w1, h1, x0, y0, x1, y1, i, j;
  int f2;
  int r, g, b, gray;
  unsigned char *dst;

  w1 = w0 / factor;
  h1 = h0 / factor;
  f2 = factor * factor * 256;

  dst = (unsigned char *)calloc(w1 * h1, sizeof(unsigned char));
  
  for (y0 = 0, y1 = 0; y1 < h1; y1++) {
    for (x0 = 0, x1 = 0; x1 < w1; x1++) {
      r = g = b = 0;
      for (i = 0; i < factor; i++) {
	for (j = 0; j < factor; j++) {
	  r += src[((y0 + j) * w0 + x0 + i) * 3];
	  g += src[((y0 + j) * w0 + x0 + i) * 3 + 1];
	  b += src[((y0 + j) * w0 + x0 + i) * 3 + 2];
	}
      }
      gray = r * 77 + g * 150 + b * 29;
      dst[y1 * w1 + x1] = gray / f2;
      x0 += factor;
    }
    y0 += factor;
  }
  return dst;
}

unsigned char *make_palm_bmp(unsigned char *rgbmap)
{
#define BMP_SIZE (160 * 160 / 2)

  unsigned char *bmp;
  int i, j, hi, lo;

  bmp = (unsigned char *)calloc(BMP_SIZE, sizeof(unsigned char));
  
  for (i = 0; i < 160; i++) {
    for (j = 0; j < 80; j++) {
      hi = (0xff - rgbmap[i * 160 + j * 2] ) & 0xf0;
      lo = (0xff - rgbmap[i * 160 + j * 2 + 1]) >> 4;
      bmp[i * 80 + j] = hi | lo;
    }
  }

  return bmp;
}
