/*
 * Hanashizume no Matsuri for Palm
 * Copyright (c) 2001 TF <tf@denpa.org>
 * All rights reserverd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: Database.c,v 1.4 2001/03/06 14:47:43 tf Exp $
 */
#include <PalmOS.h>
#include <PalmCompatibility.h>
#include "Hana.h"
#include "HanaRsc.h"

Boolean
InitDatabase(void)
{
  VoidHand h;
  DmOpenRef savedbR;
  PrefData dummyPd;
  Int i, res;
  
  dbR = DmOpenDatabaseByTypeCreator('DATA', CREATOR_ID, dmModeReadOnly);
  if(dbR == NULL){
    return false;
  }

  /* the first record is the table of the PDB file */
  h = DmQueryRecord(dbR, 0);
  if (h == NULL) {
    FrmAlert(WarningAlert);
    return false;
  }
  MemMove(&pdbTable, MemHandleLock(h), sizeof(PDBTable));
  MemHandleUnlock(h);

  /* prepate database for save, if it does not exist. */
  savedbR = DmOpenDatabaseByTypeCreator('SAVE', CREATOR_ID, dmModeReadWrite);
  if (savedbR == NULL) {
    if (DmCreateDatabase(0, SAVE_DB_NAME, CREATOR_ID, 'SAVE', false)) {
      FrmAlert(WarningAlert);
      return false;
    }
    savedbR = DmOpenDatabaseByTypeCreator('SAVE', CREATOR_ID, dmModeReadWrite);
    if (savedbR == NULL) {
      FrmAlert(WarningAlert);
      return false;
    }
    for (i = 0; i < 3; i++) {
      res = i;
      h = DmNewRecord(savedbR, &res, sizeof(PrefData));
      if (h == NULL) {
	FrmAlert(WarningAlert);
	return false;
      }
      dummyPd.CurBlock = 0;
      DmWrite(MemHandleLock(h), 0, &dummyPd, sizeof(dummyPd));
      MemHandleUnlock(h);
      DmReleaseRecord(savedbR, res, true);
    }
  }
  DmCloseDatabase(savedbR);

  return true;
}

Boolean
SaveGame(UInt id)
{
  DmOpenRef savedbR;
  VoidHand h;
  
  savedbR = DmOpenDatabaseByTypeCreator('SAVE', CREATOR_ID, dmModeReadWrite);
  if (savedbR == NULL) {
    FrmAlert(WarningAlert);
    return false;
  }
  h = DmGetRecord(savedbR, id);
  if (h == NULL) {
    FrmAlert(CGWarningAlert);
    return false;
  }
  DmWrite(MemHandleLock(h), 0, &pd, sizeof(pd));
  MemHandleUnlock(h);
  DmReleaseRecord(savedbR, id, true);

  DmCloseDatabase(savedbR);
  return true;
}

Boolean
LoadGame(UInt id)
{
  DmOpenRef savedbR;
  PrefData save;
  VoidHand h;
  
  savedbR = DmOpenDatabaseByTypeCreator('SAVE', CREATOR_ID, dmModeReadOnly);
  if (savedbR == NULL) {
    return false;
  }
  h = DmQueryRecord(savedbR, id);
  if (h == NULL) {
    FrmAlert(WarningAlert);
    return false;
  }
  MemMove(&save, MemHandleLock(h), sizeof(save));
  MemHandleUnlock(h);

  if (save.CurBlock == 0) { /* empty */
    DmCloseDatabase(savedbR);
    return false;
  }
  
  InitGame();
  NewPage();
  pd = save;
  
  RestoreGame();
  
  DmCloseDatabase(savedbR);
  
  return true;
}
