/*
 * Hanashizume no Matsuri for Palm
 * Copyright (c) 2001 TF <tf@denpa.org>
 * All rights reserverd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: Event.c,v 1.11 2001/04/07 07:37:26 tf Exp $
 */
#include <PalmOS.h>
#include <PalmCompatibility.h>
#include "HanaRsc.h"
#include "Hana.h"

#ifdef __GNUC__
#include "Callbacks.h"
#endif		

static void ProcessPenDownEvent(EventPtr);
static void ProcessKeyDownEvent(EventPtr);

Boolean
ProcessGame(void)
{
  Int res;

  switch (pd.state) {
  case STAT_MAIN:
    if (StrLen(textBuf)) {
      PutString(textBuf, offscn.winTXT, true);
      UpdateScreen();
    } else {
      res = GetNextToken(textBuf, false);
      switch (res) {
      case TOKEN_CMD:
	ProcessCommand(textBuf);
	break;
      case TOKEN_STR:
	PutString(textBuf, offscn.winTXT, true);
	UpdateScreen();
	break;
      case TOKEN_ALT:
	PushState(pd.state);
	pd.state = STAT_ALTN;
	SetAlternative(textBuf, offscn.winBG);
	UpdateScreen();
	if (mode & MODE_SKIP) {
	  mode &= ~ MODE_SKIP;
	}
	break;
      }
    }
    break;
  case STAT_WAIT:
  case STAT_ALTN:
  case STAT_CG:
  case STAT_EFCT:
    break;
  }

  return true;
}

Boolean
MainFormHandleEvent(EventPtr event)
{
  Boolean handled;
  // Char buf[256];
#ifdef __GNUC__
  CALLBACK_PROLOGUE
#endif
  handled = false;

  switch (event->eType) {
  case ctlSelectEvent:
    /* A control button was pressed and released. */
    FrmAlert(WarningAlert);
    handled = true;
    break;

  case penDownEvent:
#ifndef PALM35
    SetScreenDepth(4);
#endif
    ProcessPenDownEvent(event);
    handled = true;
    break;

  case keyDownEvent:
    ProcessKeyDownEvent(event);
    handled = true;
    break;

  case menuEvent:		
#ifndef PALM35
    SetScreenDepth(4);
#endif
    switch (event->data.menu.itemID) {
    case SaveItem1:
      SaveGame(0);
      break;
    case SaveItem2:
      SaveGame(1);
      break;
    case SaveItem3:
      SaveGame(2);
      break;
    case LoadItem1:
      LoadGame(0);
      break;
    case LoadItem2:
      LoadGame(1);
      break;
    case LoadItem3:
      LoadGame(2);
      break;
    case OpeningItem:
      InitGame();
      NewPage();
      pd.CurBlock = 3080;
      TextLoad(pd.CurBlock);
      pd.Pos = 0;
      break;
    case TitleItem:
      InitGame();
      NewPage();
      pd.Pos = FindStartMenu();
      break;
    case AboutItem:
#ifndef PALM35
      SetScreenDepth(0);
#endif
      FrmAlert(AboutAlert);
#ifndef PALM35
      SetScreenDepth(4);
#endif
      break;
    }
    handled = true;
    break;
  }

  if (!handled) {
    ProcessGame();
  }
  
#ifdef __GNUC__
  CALLBACK_EPILOGUE
#endif
    return true;
}

static void
ProcessPenDownEvent(EventPtr event)
{
  if (mode & MODE_SKIP) {
    mode &= ~ MODE_SKIP;
  }

#ifndef PALM35
  SetScreenDepth(4);
#endif

  switch (pd.state) {
  case STAT_MAIN:
    evtTimeout = DEFAULT_TIMEOUT;
    break;
  case STAT_WAIT:
    evtTimeout = DEFAULT_TIMEOUT;
    pd.state = PopState();
    break;
  case STAT_ALTN:
    if (event->screenY < alternative.first || event->screenY > alternative.last) {
      break;
    }
    alternative.selected = true;
    alternative.result = (event->screenY - alternative.first - 1) / HEIGHT_CH;
    pageBreak = true;
    pd.state = PopState();
    break;
  case STAT_CG:
    pd.state = PopState();
#ifdef PALM35
    WinPaintBitmap(bmpWK, 0, 0);
#endif
    break;
  }
}

static void
ProcessKeyDownEvent(EventPtr event)
{
#ifndef PALM35
  SetScreenDepth(4);
#endif
  if (mode & MODE_SKIP) {
    mode &= ~ MODE_SKIP;
    return;
  }

  switch (event->data.keyDown.chr) {
  case pageUpChr:
    switch (pd.state) {
    case STAT_WAIT:
      mode |= MODE_PAGESKIP;
      evtTimeout = DEFAULT_TIMEOUT;
      pd.state = PopState();
      break;
    }
    break;
  case pageDownChr:
    switch (pd.state) {
    case STAT_WAIT:
      evtTimeout = DEFAULT_TIMEOUT;
      pd.state = PopState();
      break;
    }
    break;
  case vchrCalc:
  case vchrKeyboard:
    if (pd.state == STAT_ALTN || pd.state == STAT_CG) {
      break;
    }
    mode |= MODE_SKIP;
    if (pd.state == STAT_WAIT) {
      pd.state = PopState();
      evtTimeout = DEFAULT_TIMEOUT;
    }
    break;
  case vchrFind:
    if (pd.state != STAT_CG) {
      PushState(pd.state);
      pd.state = STAT_CG;
      evtTimeout = evtWaitForever;
#ifdef PALM35
      WinSetDrawWindow(NULL);
      WinPaintBitmap (bmpBG, 0, 0);
#else
      MemMove(offscn.pFrameBuffer, MemHandleLock(offscn.fbBG), 160 * 160 / 2);
      MemHandleUnlock(offscn.fbBG);
#endif
    } else {
      pd.state = PopState();
      evtTimeout = DEFAULT_TIMEOUT;
#ifdef PALM35
      WinSetDrawWindow(NULL);
      WinPaintBitmap (bmpWK, 0, 0);
#else
      MemMove(offscn.pFrameBuffer, MemHandleLock(offscn.fbWK), 160 * 160 / 2);
      MemHandleUnlock(offscn.fbWK);
#endif
    }
    break;
  }
}

void
PushState(Int s)
{
  if (stackDepth == 5) {
    FrmAlert(WarningAlert); /* stack overflow */
    return;
  }
  stateStack[stackDepth] = s;
  stackDepth++;
}

Int
PopState()
{
  if (stackDepth == 0) {
    FrmAlert(WarningAlert); /* empty stack */
    return STAT_MAIN;
  }
  stackDepth--;
  return stateStack[stackDepth];
}

void
RestoreGame(void)
{
  Int res, targetBlock, targetPos;
  Boolean alt = false;

  TextLoad(pd.topBlock);

  ImageLoad(pd.CurImg);
  
  targetPos = pd.Pos;
  pd.Pos = pd.topPos;

  pd.state = pd.topState;
  targetBlock = pd.CurBlock;
  pd.CurBlock = pd.topBlock;
  CurX = CurY = 0;
  if (pd.state == STAT_ALTN) {
    alt = true;
  }

  while (pd.Pos < targetPos || pd.CurBlock != targetBlock) {
    if (StrLen(textBuf)) {
      PutString(textBuf, offscn.winTXT, true);
    } else {
      res = GetNextToken(textBuf, alt);
      alt = false;
      switch (res) {
      case TOKEN_CMD:
	switch (textBuf[0]) {
	case 'p':
	  if (textBuf[1] == 'e') {
	    pageBreak = true;
	  }
	  break;
	}
	textBuf[0] = '\0';
	break;
      case TOKEN_STR:
	PutString(textBuf, offscn.winTXT, true);
	break;
      case TOKEN_ALT:
	SetAlternative(textBuf, offscn.winTXT);
	PushState(STAT_MAIN);
	pd.state = STAT_ALTN;
	UpdateScreen();
	return;
      }
    }
  }
  UpdateScreen();
  PushState(STAT_MAIN);
  pd.state = STAT_WAIT;
  evtTimeout = evtWaitForever;
  skipTap = true;
}

void
InitGame(void)
{
  pd.CurBlock = 3080;
  TextLoad(pd.CurBlock);
  pd.Pos = 0;
  
  pd.flagMegumi = pd.flagYue = pd.flagRuria =
    pd.flagSarasa = pd.flagHinata = pd.flagHimiko =
    pd.flagMao = pd.flagKurumi = pd.flagMinako = 0;

  CurX = 0;
  CurY = 0;
  text[0] = '\0';
  textBuf[0] = '\0';
  pd.state = STAT_MAIN;
  evtTimeout = DEFAULT_TIMEOUT;
  mode = 0;
  pageBreak = false;
  skipTap = false;
  stackDepth = 0;
}

void
ChangeForm(UInt16 formId)
{
  if (currentForm == formId) {
    return;
  }
  currentForm = formId;
  FrmGotoForm(formId);
}
