/*
 * Hanashizume no Matsuri for Palm
 * Copyright (c) 2001 TF <tf@denpa.org>
 * All rights reserverd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: Graphics.c,v 1.6 2001/03/06 14:56:10 tf Exp $
 */
#include <PalmOS.h>
#include <PalmCompatibility.h>

#include "HanaRsc.h"
#include "Hana.h"

/* PalmOS prior to 3.5 doesn't support the OS-level color control.  */
Boolean
SetScreenDepth(Int newDepth)
{
#ifdef PALM35
  if (currentScreenDepth == newDepth) {
    return true;
  }

  switch (newDepth) {
  case 0:  /* default */
    {
      Boolean bColor;
      WinScreenMode(winScreenModeGet, NULL, NULL, NULL, &bColor);
      /* color device or not */
      if (bColor) {
	UIColorPopTable();
      }
      
      if (WinScreenMode(winScreenModeSetToDefaults, NULL, NULL, NULL, &bColor) == 0 ) {
	currentScreenDepth = 0;
      } else {
	return false;
      }
    }
  break;
  case 2: /* 2bpp */
    {    
      UInt32 depth = 2;
      Boolean bColor = true;
      if (WinScreenMode(winScreenModeSet, NULL, NULL, &depth, &bColor) == 0)
	currentScreenDepth = 2;
      else
	return false;
      break;
    }
  case 4:
    {
      UInt32 depth = 4;
      static RGBColorType rgbs[16];
      int i;
      Boolean bColor;
      
      /* check if color device or not */
      if (WinScreenMode(winScreenModeGet, NULL, NULL, NULL, &bColor) != 0)
	return false;
      
      if (bColor) {
	UIColorPushTable();
	for (i = 0; i < 16; i++) {
	  rgbs[i].index = i;
	  rgbs[i].r = 255-i*16;
	  rgbs[i].b = 255-i*16;
	  rgbs[i].g = 255-i*16;
	}
	rgbs[15].r = rgbs[15].b = rgbs[15].g = 0x0; /* Set to white */
      }
      
      if (WinScreenMode(winScreenModeSet, NULL, NULL, &depth, &bColor) == 0) {
	currentScreenDepth = 4;
      } else {
	return false;
      }
      
      if (bColor) {
	WinPalette(winPaletteSet, 0, 16, rgbs);
      }
      break;
    }
  default:
    ErrFatalDisplay("SetScreenDepth("__FILE__") : Illegal screen depth.");
    return false;
  }
#else
  volatile unsigned char* LVPW = (unsigned char* )0xfffffa05;
  volatile unsigned char* LPICF = (unsigned char* )0xfffffa20;
  volatile unsigned char* CKCON = (unsigned char *)0xfffffa27;
  volatile unsigned char* FRCM = (unsigned char *)0xfffffa31;
  volatile unsigned char* LPXCD = (unsigned char* )0xfffffa25;
  static unsigned char oldLPXCD;
  const unsigned short* volatile * SSA = (const unsigned short**)0xfffffa00;
  const static unsigned short* oldSSA;
  static unsigned char oldLVPW;
  static unsigned char oldLPICF;
  static unsigned char oldFRCM;

  if (currentScreenDepth == newDepth) {
    return true;
  }

  *CKCON &= 0x7f; /* off the LCD */

  switch (newDepth) {
  case 0:
    *LPICF = oldLPICF;
    *LVPW = oldLVPW;
    *FRCM = oldFRCM;
    *SSA = oldSSA; /* restore the framebuffer */
    *LPXCD = oldLPXCD;
    currentScreenDepth = 0;
    break;
  case 4:
    oldSSA = *SSA;
    *SSA = (const unsigned short *)offscn.pFrameBuffer;
    oldLVPW = *LVPW;
    *LVPW = 40;
    oldLPICF = *LPICF;
    *LPICF = (*LPICF & 0xfc) | 0x02; /* specify by the lower 2 bits  */
    oldFRCM = *FRCM;
    *FRCM = 0xb9;
    currentScreenDepth = 4;
    // Ĥɻ?
    oldLPXCD = *LPXCD;
    *LPXCD = 0;
    break;
  default:
    ErrFatalDisplay("SetScreenDepth("__FILE__") : illigal screen depth.");
  }
  
  SysTaskDelay(8); /* wait until changes are reflected (is this delay enough?) */
  *CKCON |= 0x80;
#endif
  return true;
}

Boolean
PrepareOffscreen(void)
{
  Err err;

#ifdef PALM35
  bmpBG = BmpCreate(160, 160, 4, NULL, &err);
  bmpBG2 = BmpCreate(160, 160, 4, NULL, &err);
  bmpWK = BmpCreate(160, 160, 4, NULL, &err);
  bmpTXT = BmpCreate(160, 160, 4, NULL, &err);
  if (bmpBG == NULL || bmpBG2 == NULL || bmpWK == NULL || bmpTXT == NULL) {
    return false;
  }
  offscn.winBG = WinCreateBitmapWindow(bmpBG, &err);
  offscn.winWK = WinCreateBitmapWindow(bmpWK, &err);
  offscn.winTXT = WinCreateBitmapWindow(bmpTXT, &err);
#else
  offscn.pFrameBuffer = MemPtrNew(160 * 160 / 2);
  if (offscn.pFrameBuffer == 0) {
    return false;
  }
  MemSet(offscn.pFrameBuffer, 160 * 160 / 2, 0);
  
  offscn.fbBG = CreateOffscreenBuffer(160, 160, 2);
  offscn.fbBG2 = CreateOffscreenBuffer(160, 160, 2);
  offscn.fbWK = CreateOffscreenBuffer(160, 160, 2);
  
  offscn.winTXT = WinCreateOffscreenWindow(160, 160, genericFormat, &err);
  
  WinSetDrawWindow(offscn.winTXT);
  WinEraseWindow();
#endif

  return true;
}

VoidHand
CreateOffscreenBuffer(Int width, Int height, Int bpp)
{
  VoidHand ret;

  width /= bpp;
  width = ((width + 1) / 2) * 2;
  ret = MemHandleNew(width * height);
  if (ret == NULL) {
    FrmAlert(WarningAlert);
  }
  return ret;
}

void
CleanOffscreen(void)
{
#ifndef PALM35
  if (offscn.pFrameBuffer) {
    MemPtrFree(offscn.pFrameBuffer);
  }
#endif
}

void
UpdateScreen(void)
{
  ImageMerge();
#ifdef PALM35
  WinSetDrawWindow(NULL);
  WinPaintBitmap(bmpWK, 0, 0);
#else
  MemMove(offscn.pFrameBuffer, MemHandleLock(offscn.fbWK), 160 * 160 / 2);
  MemHandleUnlock(offscn.fbWK);
#endif
}

void
ClearScreen(void)
{
#ifndef PALM35
  MemSet(offscn.pFrameBuffer, 160 * 160 / 2, 0);
#endif
}

void
ClearTextWindow(void)
{
  UChar *p;

#ifdef PALM35
  p = BmpGetBits(bmpTXT);
  MemSet(p, 80 * 160, 0);
#else
  p = (UChar *)offscn.winTXT->displayAddrV20;
  MemSet(p, 160 * 160 / 8, 0);
#endif
}
