/*
 * Hanashizume no Matsuri for Palm
 * Copyright (c) 2001 TF <tf@denpa.org>
 * All rights reserverd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: Hana.c,v 1.6 2001/03/06 14:47:43 tf Exp $
 */
#include <PalmOS.h>
#include <PalmCompatibility.h>
#include "HanaRsc.h"

#define MAIN
#include "Hana.h"

static Err StartApplication(void);
static void StopApplication(void);
static Boolean ApplicationHandleEvent(EventPtr);
static void EventLoop(void);
static void InitVariables(void);

static Err
StartApplication(void)
{
  ChangeForm(HanaForm);

  if (!PrepareOffscreen()) {
    return 1;
  }
  SetScreenDepth(4);

  InitVariables();
  
  return 0;
}

static void
StopApplication(void)
{
  CleanOffscreen();

  /* restore screen depth */
  if (currentScreenDepth != 0) {
    SetScreenDepth(0);
  }
}

static Boolean
ApplicationHandleEvent(EventPtr event)
{
  FormPtr frm;
  Int formId;
  Boolean handled = false;

  if (event->eType == frmLoadEvent) {
    /* Load the form resource specified in the event then activate the form. */
    formId = event->data.frmLoad.formID;
    frm = FrmInitForm(formId);
    FrmSetActiveForm(frm);
    
    /*
     * Set the event handler for the form.  The handler of the currently 
     * active form is called by FrmDispatchEvent each time it receives an event.
     */
    switch (formId) {
    case HanaForm:
      FrmSetEventHandler(frm, MainFormHandleEvent);
      break;
    }
    handled = true;
  }
  
  return handled;
}

static void
EventLoop(void)
{
  EventType event;
  Word error;

  do {
    Boolean sys = true;
    EvtGetEvent(&event, evtTimeout);

    if (event.eType == keyDownEvent) {
      switch (event.data.keyDown.chr) {
      case vchrMenu:
#ifndef PALM35
	if (currentScreenDepth == 4) {
	  SetScreenDepth(0);
	} else {
	  SetScreenDepth(4);
	}
#endif
        sys = true;
        break;
      case vchrCalc:
      case vchrKeyboard:
      case vchrFind:
      case pageUpChr:
      case pageDownChr:
        sys = false;
        break;
      }
    }

    if (sys) {
      if (SysHandleEvent(&event)) {
	continue;
      }
    }

    if (!MenuHandleEvent(0, &event, &error))
      if (!ApplicationHandleEvent(&event))
	FrmDispatchEvent(&event);

  } while (event.eType != appStopEvent);
}

static void
InitVariables(void)
{
  Boolean exist;
  
  InitGame();
  
  /*
   * Preference data overrides initial data.
   */
  exist = PrefGetAppPreferencesV10(CREATOR_ID, 1, &pd, sizeof(pd)); 
  if (exist) {
    RestoreGame();
  } else {
    pd.flagClear = 0x0;
  }
}

DWord
PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags)
{
  Err err = 0;		

  if(cmd != sysAppLaunchCmdNormalLaunch) {
    return 0;
  }

  if (!InitDatabase()) {
    FrmAlert(WarningAlert);
    StopApplication();
  }

  if ((err = StartApplication()) == 0) {
    EventLoop();
    StopApplication();
  }

  PrefSetAppPreferencesV10(CREATOR_ID, 1, &pd, sizeof(pd));

  DmCloseDatabase(dbR); 

  return err;
}
