/*
 * Hanashizume no Matsuri for Palm
 * Copyright (c) 2001 TF <tf@denpa.org>
 * All rights reserverd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: Hana.h,v 1.11 2001/06/07 15:09:44 tf Exp $
 */
#ifndef __INCLUDE_HANA_H
#define __INCLUDE_HANA_H

#define DEFAULT_TIMEOUT 1

#define RINGBUF_SIZE 0x1000    /* size of ring buffer */
#define TEXTBUF_SIZE 256 /* buffer size of text parser */
#define MAX_IMGSIZE 12624
#define MAX_TEXTSIZE 7650

#define CG_NORM 0x0
#define CG_DIFF 0x1

#define CREATOR_ID 'HTMK'
#define SAVE_DB_NAME "SaveData-HTMK"

#define WIDTH_CH (5)
#define HEIGHT_CH (10)
#define WIDTH_WC (9)

#define DATABASE_SIZE 1235

#define STAT_MAIN 0
#define STAT_WAIT 1
#define STAT_ALTN 2
#define STAT_CG 3
#define STAT_EFCT 4

#define TOKEN_CMD 0
#define TOKEN_STR 1
#define TOKEN_ALT 2

#define DEBUG 1

#define MODE_SKIP (0x1)
#define MODE_PAGESKIP (0x1 << 1)
#define MODE_REDRAW (0x1 << 2)

/*
 * for status flags
 */
#define FLAG_CLEAR_MEGUMI   0x1
#define FLAG_CLEAR_YUE     (0x1 << 1)
#define FLAG_CLEAR_RURIA   (0x1 << 2)
#define FLAG_CLEAR_KURUMI  (0x1 << 3)
#define FLAG_CLEAR_HIMIKO  (0x1 << 4)
#define FLAG_CLEAR_HINATA  (0x1 << 5)
#define FLAG_CLEAR_MAO     (0x1 << 6)
#define FLAG_CLEAR_SARASA  (0x1 << 7)
#define FLAG_CLEAR_MINAKO  (0x1 << 8)
#define FLAG_CLEAR_ALL     (0x1ff)

#define MEGUMI1 (0x1)
#define MEGUMI2 (0x1 << 1)
#define MEGUMI3 (0x1 << 2)

#define YUE1 (0x1)
#define YUE2 (0x1 << 1)
#define YUE3 (0x1 << 2)
#define YUE4 (0x1 << 3)
#define YUE5 (0x1 << 4)

#define RURIA1 (0x1)
#define RURIA2 (0x1 << 1)
#define RURIA3 (0x1 << 2)
#define RURIA4 (0x1 << 3)
#define RURIA5 (0x1 << 4)

#define RURIAa (0x1 << 14)

#define SARASA1 (0x1)
#define SARASA2 (0x1 << 1)
#define SARASA3 (0x1 << 2)
#define SARASA4 (0x1 << 3)

#define SARASAa (0x1 << 4)

#define KURUMI1 (0x1)
#define KURUMI2 (0x1 << 1)
#define KURUMI3 (0x1 << 2)
#define KURUMI4 (0x1 << 3)
#define KURUMI5 (0x1 << 4)
#define KURUMI6 (0x1 << 5)

#define KURUMIa (0x1 << 10)
#define KURUMIb (0x1 << 11)

#define MINAKO1 (0x1)

#define HINATA1 (0x1)
#define HINATA2 (0x1 << 1)
#define HINATA3 (0x1 << 2)
#define HINATA4 (0x1 << 3)
#define HINATA5 (0x1 << 4)

#define HINATAa (0x1 << 10)
#define HINATAb (0x1 << 11)

#define HIMIKO1 (0x1)
#define HIMIKO2 (0x1 << 1)
#define HIMIKO3 (0x1 << 2)
#define HIMIKO4 (0x1 << 3)
#define HIMIKO5 (0x1 << 4)
#define HIMIKO6 (0x1 << 5)
#define HIMIKO7 (0x1 << 6)

#define HIMIKOa (0x1 << 10)
#define HIMIKOb (0x1 << 11)
#define HIMIKOc (0x1 << 12)

#define MAO1 (0x1)
#define MAO2 (0x1 << 1)
#define MAO3 (0x1 << 2)
#define MAO4 (0x1 << 3)

typedef struct {
  UInt16 len[DATABASE_SIZE];
} PDBTable;

typedef struct {
  Boolean selected;
  int result;
  UChar first;
  UChar last;
} Alternative;

typedef struct {
  Int evtTimeout;
  Int state, mode, topState;
  Boolean pageBreak, skipTap;
  Int CurImg;
  Int CurBlock, Pos, topBlock, topPos;
  Int flagClear;
  Int flagMegumi, flagYue, flagRuria, flagSarasa;
  Int flagHinata, flagHimiko, flagMao, flagKurumi, flagMinako;
} PrefData;

#ifdef PALM35
typedef struct {
  WinHandle winBG, winTXT, winWK;
} OffscreenData;
#else
typedef struct {
  UChar *pFrameBuffer;
  WinHandle winBG, winTXT, winWK;
  VoidHand fbBG, fbBG2, fbWK;
} OffscreenData;
#endif

/* Event.c */
extern Boolean MainFormHandleEvent(EventPtr);
extern void PushState(Int);
extern Int PopState(void);
extern void RestoreGame(void);
extern void InitGame(void);
extern void ChangeForm(UInt16);

/* Database.c */
extern Boolean InitDatabase(void);
extern Boolean SaveGame(UInt);
extern Boolean LoadGame(UInt);

/* Graphics.c */
extern Boolean SetScreenDepth(Int);
extern void UpdateScreen(void);
extern void ClearTextWindow(void);
extern Boolean PrepareOffscreen(void);
extern void CleanOffscreen(void);
#ifndef PALM35
extern VoidHand CreateOffscreenBuffer(Int, Int, Int);
#endif

/* Image.c */
extern void ImageLoad(UInt32);
extern void ImageString(WinHandle, const UChar *, Coord, Coord);
extern void ImageMerge(void);
extern Int DecodeLZ77(UChar *, UChar *, Int);

/* Text.c */
extern void TextLoad(UInt32);
extern void PutString(UChar *, WinHandle, Boolean);
extern void NewPage(void);
extern void SetAlternative(UChar *, WinHandle);
extern Int FindStartMenu(void);

/* Script.c */
extern Int GetNextToken(UChar *, Boolean);
extern void ProcessCommand(UChar *);
extern void CommandPause(void);

#ifdef MAIN
#define EXTERN
#else
#define EXTERN extern
#endif

/* to be saved */
EXTERN Int evtTimeout;
EXTERN Int state, mode;
EXTERN Boolean pageBreak, skipTap;
EXTERN PrefData pd;

/* need not to be saved */
EXTERN Int currentScreenDepth;
EXTERN Int currentForm;
EXTERN OffscreenData offscn;
#ifdef PALM35
EXTERN BitmapType *bmpBG, *bmpBG2, *bmpTXT, *bmpWK;
#else
EXTERN BitmapType *bmpBG, *bmpBG2, *bmpWK;
#endif
EXTERN DmOpenRef dbR;
EXTERN PDBTable pdbTable;
EXTERN Int CurX, CurY;
// EXTERN Int Pos, topPos;
EXTERN Int topPos;
EXTERN UChar text[MAX_TEXTSIZE];
EXTERN UChar textBuf[TEXTBUF_SIZE];
EXTERN Alternative alternative;
EXTERN Int stateStack[5], stackDepth;
EXTERN UChar dbimg[MAX_IMGSIZE];
#endif
