/*
 * Hanashizume no Matsuri for Palm
 * Copyright (c) 2001 TF <tf@denpa.org>
 * All rights reserverd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: Image.c,v 1.9 2001/06/07 15:09:44 tf Exp $
 */
#include <PalmOS.h>
#include <PalmCompatibility.h>

#include "HanaRsc.h"
#include "Hana.h"

UChar ring[RINGBUF_SIZE];

/*
 * Load an image into "background" image buffer.
 */
void
ImageLoad(UInt32 id)
{
  VoidHand h;
  UChar *p, *q;
  UInt16 idx, i;
  Err err;
  static Int bg2_id;

#ifdef PALM35
  if (!offscn.winBG) {
    return;
  }
  WinSetDrawWindow(offscn.winBG);
#endif

  /* load image from database */
  err = DmFindRecordByID(dbR, id, &idx);
  if (err) {
    FrmAlert(CGWarningAlert);
    return;
  }
  h = DmQueryRecord(dbR, idx);
  if (h == NULL) {
    FrmAlert(CGWarningAlert);
    return;
  }
  MemMove(&dbimg, MemHandleLock(h), pdbTable.len[idx - 1]);
  MemHandleUnlock(h);

#ifdef PALM35
  p = BmpGetBits(bmpBG);
#else
  p = (UChar *)MemHandleLock(offscn.fbBG);
#endif
  if (dbimg[0] == CG_NORM) {
    DecodeLZ77(p, dbimg + 1, pdbTable.len[idx - 1] - 1);
  } else { /* CG_DIFF */
    Int arg;
    DecodeLZ77(p, dbimg + 3, pdbTable.len[idx - 1]);

    arg = (dbimg[1] << 8) | dbimg[2];
#ifdef PALM35
    q = BmpGetBits(bmpBG2);
#else
    q = (UChar *)MemHandleLock(offscn.fbBG2);
#endif
    if (arg != bg2_id) {
      err = DmFindRecordByID(dbR, arg, &idx);
      if (err) {
	FrmAlert(CGWarningAlert);
	return;
      }
      h = DmQueryRecord(dbR, idx);
      if (h == NULL) {
	FrmAlert(CGWarningAlert);
	return;
      }
      MemMove(&dbimg, MemHandleLock(h), pdbTable.len[idx - 1]);
      MemHandleUnlock(h);
      if (dbimg[0] != CG_NORM) { /* CG_DIFF can refer only CG_NORM. */
	FrmAlert(CGWarningAlert);
	return;
      }
      DecodeLZ77(q, dbimg + 1, pdbTable.len[idx - 1] - 1);
      bg2_id = arg;
    }
    for (i = 0; i < 80 * 160; i++) {
      p[i] = p[i] ^ q[i];
    }
#ifndef PALM35
    MemHandleUnlock(offscn.fbBG2);
#endif
  }
#ifndef PALM35
  MemHandleUnlock(offscn.fbBG);
#endif
  pd.CurImg = id;
}

void ImageString(WinHandle w, const UChar *chars, Coord x, Coord y)
{
  Int16 len;

  if (!w) {
    return;
  }
  WinSetDrawWindow(w);
  
  len = StrLen(chars);

  WinDrawChars(chars, len, x, y);
}

void
ImageMerge(void)
{
  UChar *p, *q, *r;
  int i, j;
#ifndef PALM35
  int u, l;
#endif

#ifdef PALM35
  if (!offscn.winWK) {
    return;
  }

  p = BmpGetBits(bmpBG);
  q = BmpGetBits(bmpTXT);
  r = BmpGetBits(bmpWK);
  for (i = 0; i < 80 * 160; i++) {
    *r++ = ((*p++ >> 2) & 0x33) | *q++;
  }
#else
  p = (UChar *)MemHandleLock(offscn.fbBG);
  q = (UChar *)offscn.winTXT->displayAddrV20;
  r = (UChar *)MemHandleLock(offscn.fbWK);

  for (i = 0; i < 80 * 160; i++) {
    u = (p[i] & 0xf0) * 3;
    l = (p[i] & 0x0f) * 3;
    r[i] = ((u >> 2) & 0xf0) | ((l >> 2) & 0x0f);
  }
  MemHandleUnlock(offscn.fbBG);
  
  for (i = 0; i < 160; i++) {
    for (j = 0; j < 160 / 8; j++) {
      if (*q >> 7 & 0x1) {
	*r |= 0xf0;
      }
      if (*q >> 6 & 0x1) {
	*r |= 0x0f;
      }
      r++;
      
      if (*q >> 5 & 0x1) {
	*r |= 0xf0;
      }
      if (*q >> 4 & 0x1) {
	*r |= 0x0f;
      }
      r++;
      
      if (*q >> 3 & 0x1) {
	*r |= 0xf0;
      }
      if (*q >> 2 & 0x1) {
	*r |= 0x0f;
      }
      r++;
      
      if (*q >> 1 & 0x1) {
	*r |= 0xf0;
      }
      if (*q & 0x1) {
	*r |= 0x0f;
      }
      r++;
      
      q++;
    }
  }
  MemHandleUnlock(offscn.fbWK);
#endif
}

Int
DecodeLZ77(UChar *dst, UChar *src, Int size)
{
  Int i, j, k;
  Int c, m;
  Int flag = 0;
  Int upper, lower;
  Int pos, len;
  Int idx = 0;
  
  /* initialize ring buffer */
  for (i = 0; i < RINGBUF_SIZE; i++){
    ring[i] = 0;
  }

  /* extract data */
  for (c = 0, m = 0xfee, i = 0, k = 0; idx < 80 * 160;){
    /* flag bits, which indicates data or location */
    if (--c < 0) {
      flag = src[k++];
      c = 7;
    }
    
    if(flag & 0x80){
      /* data */
      ring[m++] = dst[idx++] = src[k++];
      m &= 0xfff;
    } else {
      /* copy from ring buffer */
      upper = src[k++];
      lower = src[k++];
      len = (upper & 0x0f) + 3;
      pos = (upper >> 4) + (lower << 4);

      for(j = 0; j < len; j++){
	ring[m++] = dst[idx++] =  ring[pos++];
	m &= 0xfff;
	pos &= 0xfff;
      }
    }
    flag = flag << 1;
    if (k >= size) {
      break;
    }
  }
  return idx;
}
