/*
 * Hanashizume no Matsuri for Palm
 * Copyright (c) 2001 TF <tf@denpa.org>
 * All rights reserverd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: Script.c,v 1.12 2001/06/07 15:09:44 tf Exp $
 */
#include <PalmOS.h>
#include <PalmCompatibility.h>
#include "HanaRsc.h"
#include "Hana.h"

static Int FiveDigitToInt(const Char *);
static Int NextBlock(Int, Boolean *);
static void CommandPageEnd(void);
static void CommandImage(UChar *);

Int
GetNextToken(UChar *buf, Boolean alt)
{
  Int i = 0;
  Boolean isAlt = false;

  if (text[pd.Pos] == 0x0) {
    pd.CurBlock = NextBlock(pd.CurBlock, &isAlt);
    TextLoad(pd.CurBlock);
  }
  if (alt) {
    isAlt = true;
  }
  
  topPos = pd.Pos;

  if (text[pd.Pos] >= 0x20 && text[pd.Pos] <= 0x7f) { /* JIS X 0201 LH */
    /* command */
    while (text[pd.Pos] < 0x80) {
      if (text[pd.Pos] == 0x5c && text[pd.Pos + 1] == 0x0a) { /* '<backslash><LF>' */
	pd.Pos += 2;
	break;
      }
      if (text[pd.Pos] == 0x0a || text[pd.Pos] == 0x0) {
	break;
      }
      buf[i++] = text[pd.Pos++];
    }
    buf[i] = '\0';
    return TOKEN_CMD;
  } else {
    /* string */
    while (1) {
      if (i > TEXTBUF_SIZE - 3) {
	break;
      }
      if (text[pd.Pos] == 0x5c && text[pd.Pos + 1] == 0x0a) { /* '<backslash><LF>' */
	pd.Pos += 2;
	break;
      }
      if (text[pd.Pos] == 0x0) {
	break;
      }
      if (text[pd.Pos] > 0x1f && text[pd.Pos] < 0x80) { /* switch to command */
	break;
      } else if (text[pd.Pos] < 0x20) { /* control char. */
	buf[i++] = text[pd.Pos++];
      } else { /* double-byte char */
	buf[i++] = text[pd.Pos++];
	buf[i++] = text[pd.Pos++];
      }
    }
    buf[i] = '\0';
    if (isAlt) {
      return TOKEN_ALT;
    } else {
      return TOKEN_STR;
    }
  }
}

void
ProcessCommand(UChar *cmd)
{
  switch(cmd[0]) {
  case 'p':
    if (cmd[1] == 'e') {
      CommandPageEnd();
    } else {
      CommandPause();
    }
    break;
  case 'a':
  case 'z':
    CommandImage(cmd);
    break;
  case 'w':
    /* to be implemented... */
    break;
  case ' ':
  case 'e':
  case 'n':
  case 'o':
  case 'r':
  case 'v':
    /* ignored */
    break;
  default:
  }
  cmd[0] = '\0';
}

void
CommandPause(void)
{
  if (mode & MODE_SKIP || mode & MODE_PAGESKIP) {
    return;
  }

  PushState(pd.state);
  pd.state = STAT_WAIT;
  evtTimeout = evtWaitForever;
  skipTap = true;
}

static void
CommandPageEnd(void)
{
  pageBreak = true;
  
  mode &= ~MODE_PAGESKIP;
  
  if (mode & MODE_SKIP) {
    return;
  }
  
  PushState(pd.state);
  pd.state = STAT_WAIT;
  evtTimeout = evtWaitForever;
}

static void
CommandImage(UChar *cmd)
{
  Int i = 0, j, id;
  UChar buf[10];

  cmd++;

  if (cmd[0] == 'C' && cmd[1] == 'G') {
    cmd += 2;
  }

  while (*(cmd + i) != '\0' && *(cmd + i) != '/') {
    i++;
  }
  
  for (j = 0; j < i; j++) {
    buf[j] = *(cmd + j);
  }
  buf[j] = '\0';

  if (buf[0] == 'H' && buf[1] == 'A' && buf[2] == 'N') { /* HANA */
    id = 2499;
  } else if (buf[0] == 'B' && buf[1] == 'L' && buf[2] == 'A') { /* BLACK */
    id = 2498;
  } else if (buf[0] == 'h' && buf[1] == 'n' && buf[5] == '1') { /* hnti01 */
    id = 2497;
  } else if (buf[0] == 'h' && buf[1] == 'n' && buf[5] == '2') { /* hnti02 */
    id = 2496;
  } else if (buf[0] == 'h' && buf[1] == 'n' && buf[5] == '3') { /* hnti03 */
    id = 2495;
  } else if (buf[0] == 'h' && buf[1] == 'n' && buf[5] == '4') { /* hnti04 */
    id = 2494;
  } else if (buf[0] == 'h' && buf[1] == 'n' && buf[5] == '5') { /* hnti05 */
    id = 2493;
  } else {
    id = FiveDigitToInt(buf);
  }
  
  ImageLoad(id);
  UpdateScreen();
}

static Int
FiveDigitToInt(const Char *str)
{
  Int res = 0, i;
  Int base = 10000;

  for (i = 0; i < 5; i++) {
    res += (str[i] - 0x30) * base;
    base /= 10;
  }

  return res;
}

static Int
NextBlock(Int block, Boolean *isAlt)
{
  pd.Pos = 0;
  
  switch (block) {
    /* start */
  case 0:
    return 3080;
    /* block 0x7 */
  case 2645:
    return 2646;
  case 2646:
    *isAlt = true;
    if (pd.flagClear & FLAG_CLEAR_MEGUMI) {
      return 2648;
    } else {
      return 2647;
    }
  case 2647:
    switch (alternative.result) {
    case 0: /* Ƥӵ */
      return 2649;
    case 1: /* 餫Τ */
      return 2650;
    }
  case 2648:
    switch (alternative.result) {
    case 0: /* Ƥӵ */
      return 2649;
    case 1: /* 餫Τ */
      return 2650;
    case 2: /* Τǿ³ */
      pd.flagYue |= YUE1;
      pd.flagRuria |= RURIA1;
      return 2651;
    }
  case 2649:
    return 3085;
  case 2650:
    return 3085;
  case 2651:
    return 3085;
  case 2653:
    switch (alternative.result) {
    case 0: /* й */
      if (pd.flagYue & YUE1 || pd.flagRuria & RURIA1) {
	pd.flagYue |= YUE2;
	pd.flagRuria |= RURIA2;
      }  else {
	pd.flagSarasa |= SARASA1;
	pd.flagMegumi |= MEGUMI1;
      }
      return 2654;
    case 1: /* ᤰߤٽΤԤ */
      pd.flagHinata |= HINATA1;
      return 2655;
    }
  case 2654:
    pd.flagSarasa |= SARASAa;
    return 2660;
  case 2655:
    return 2666;
    /* block 0x8 */
  case 2660:
    *isAlt = true;
    return 3240;
  case 2661:
    return 2663;
  case 2662:
    return 2663;
  case 2663:
    if (pd.flagYue & YUE2) {
      return 2664;
    } else {
      return 2665; /*  */
    }
  case 2664:
    return 2667;
  case 2665:
    return 2667;
  case 2666:
    return 2667;
  case 2667:
    if (pd.flagYue & YUE2) {
      return 2669;
    } else if (pd.flagSarasa & SARASAa) {
      return 2668;
    } else {
      return 2674;
    }
  case 2668:
    return 2674;
  case 2669:
    *isAlt = true;
    return 3241;
  case 2670:
    return 2673;
  case 2671:
    return 2673;
  case 2672:
    return 2673;
  case 2673:
    return 2683;
  case 2674:
    if ((pd.flagClear & FLAG_CLEAR_MEGUMI) &&
	(pd.flagClear & FLAG_CLEAR_YUE) &&
	(pd.flagClear & FLAG_CLEAR_RURIA) &&
	(pd.flagClear & FLAG_CLEAR_KURUMI) &&
	(pd.flagClear & FLAG_CLEAR_HIMIKO) &&
	(pd.flagSarasa & SARASA1)) {
      return 2675;
    } else {
      return 2680;
    }
  case 2675:
    *isAlt = true;
    return 3242;
    /* block 0x9 */
  case 2680:
    return 2682;
  case 2681:
    return 2682;
  case 2682:
    *isAlt = true;
    return 2683;
  case 2683:
    switch (alternative.result) {
    case 0: /* ᤰߤ˿⻲äˤ */
      return 2684;
    case 1: /* ˤĤƤҤʤʹ˹Ԥ */
      if (pd.flagHinata & HINATA1) {
	pd.flagHinata |= HINATAb;
	return 2690;
      } else {
	return 2689;
      }
    }
  case 2684:
    return 2685;
  case 2685:
    return 2687;
  case 2687:
    return 2691;
  case 2689:
    return 2691;
  case 2690:
    return 2691;
  case 2691:
    if (pd.flagSarasa & SARASA2) {
      return 2692;
    } else if (pd.flagYue & YUE2 || pd.flagRuria & RURIA2) {
      return 2701;
    } else {
      return 2700;
    }
  case 2692:
    *isAlt = true;
    return 3243;
  case 2694:
    return 2700;
  case 2695:
    return 9999;
    /* block 0xa */
  case 2700: /* եå&åץ */
    return 2702;
  case 2701: /* ɥˡѥ */
    return 2702;
  case 2702:
    if (pd.flagYue & YUE2) {
      return 2704;
    } else {
      return 2703;
    }
  case 2703:
    return 2705;
  case 2704:
    return 2705;
  case 2705:
    return 2706;
  case 2706:
    return 2707;
  case 2707:
    return 2708;
  case 2708:
    *isAlt = true;
    if (pd.flagClear & FLAG_CLEAR_YUE) {
      if (pd.flagHinata & HINATA1) {
	return 3245;
      } else {
	return 3244;
      }
    } else {
      return 3244;
    }
  case 2709:
    if (pd.flagYue & YUE2) {
      return 2711;
    } else {
      return 2710;
    }
  case 2710:
    return 2714;
  case 2711:
    *isAlt = true;
    return 3246;
  case 2712:
    return 2714;
  case 2713:
    return 2714;
  case 2714:
    if (pd.flagYue & YUE3 || pd.flagRuria & RURIA3) {
      return 2727;
    } else if (pd.flagSarasa & SARASA3) {
      return 2726;
    } else {
      return 2725;
    }
  case 2715:
    if (pd.flagRuria & RURIA2) {
      return 2711;
    } else {
      return 3345;
    }
    /* block 0xb */
  case 2720:
    *isAlt = true;
    return 3247;
  case 2722:
    return 2723;
  case 2723:
    if (pd.flagHinata & HINATAb) {
      return 2724;
    } else {
      return 2725;
    }
  case 2724:
    return 2728;
  case 2725:
    return 2728;
  case 2726:
    return 2728;
  case 2727:
    return 2728;
  case 2728:
    *isAlt = true;
    return 3250;
  case 2729:
    return 2731;
  case 2731:
    if (pd.flagYue & YUE3 || pd.flagRuria & RURIA3) {
      return 3093;
    } else if (pd.flagSarasa & SARASAa) {
      return 2733;
    } else {
      return 2732;
    }
  case 2732:
    return 2735;
  case 2733:
    return 2735;
  case 2735:
    *isAlt = true;
    return 3251;
    /* block 0xc */
  case 2740:
    if (pd.flagYue & YUE2) {
      return 2741;
    } else if ((pd.flagClear & FLAG_CLEAR_MEGUMI) &&
	       (pd.flagKurumi & KURUMI1 || pd.flagHinata & HINATA1)) {
      return 2742;
    } else {
      return 2741;
    }
  case 2741:
    return 2768;
  case 2742:
    return 2768;
  case 2743: /* ending */
    return 3066;
    /* block 0xd */
  case 2768:
    return 2769;
  case 2769:
    if (pd.flagSarasa & SARASA3) {
      return 2770;
    } else {
      return 2772;
    }
  case 2770:
    return 2785;
  case 2771:
    return 3020;
  case 2772:
    *isAlt = true;
    if (pd.flagHinata & HINATAb) {
      return 2774;
    } else {
      return 2773;
    }
  case 2773:
    switch (alternative.result) {
    case 0: /* ΢夯ޤǲ̤򤹤 */
      if (pd.flagKurumi & KURUMI1) {
	pd.flagKurumi |= KURUMI2;
      }
      pd.flagKurumi |= KURUMIa;
      pd.flagKurumi |= KURUMIb;
      return 3095;
    case 1: /* ᤰߤݤ뤫Ƥ */
      pd.flagKurumi |= KURUMIa;
      pd.flagKurumi |= KURUMIb;
      return 2782;
    }
  case 2774:
    switch (alternative.result) {
    case 0: /* ΢夯ޤǲ̤򤹤 */
      pd.flagKurumi |= KURUMIa;
      pd.flagKurumi |= KURUMIb;
      return 3095;
    case 1: /* ᤰߤݤ뤫Ƥ */
      pd.flagKurumi |= KURUMIa;
      pd.flagKurumi |= KURUMIb;
      return 2782;
    case 2: /* ر༷Ի׵ġֶ̤ޤä򤹤 */
      return 2783;
    }
    /* block 0xe */
  case 2780:
    return 2781;
  case 2781:
    return 3101;
  case 2782:
    if (pd.flagKurumi & KURUMI1) {
      return 2780;
    } else {
      return 2781;
    }
  case 2783:
    if (pd.flagHinata & HINATA1 || pd.flagHimiko & HIMIKO1) {
      return 3103;
    } else {
      return 2784;
    }
  case 2784:
    return 2785;
  case 2785:
    return 3020;
  case 2787:
    return 3020;
  case 2788:
    return 3020;
  case 2790:
    return 2791;
  case 2791:
    return 3111;
  case 2792:
    *isAlt = true;
    return 3254;
  case 2793:
    return 3111;
  case 2794:
    return 3111;
    /* block 0xf */
  case 2801:
    return 2803;
  case 2803:
    return 2804;
  case 2804:
    if (pd.flagKurumi & KURUMI3) {
      return 2808;
    } else {
      return 3113;
    }
  case 2807:
    return 2808;
  case 2808:
    return 2809;
  case 2809:
    return 2810;
  case 2810:
    *isAlt = true;
    return 3260;
  case 2811:
    if (pd.flagKurumi & KURUMI3) {
      return 2813;
    } else {
      return 2812;
    }
  case 2812:
    return 2814;
  case 2813:
    return 2814;
  case 2814:
    return 2822;
  case 2815: /* ending */
    return 3066;
    /* block 0x10 */
  case 2820:
    if (pd.flagHimiko & HIMIKO1) {
      return 2821;
    } else {
      return 2822;
    }
  case 2821:
    return 2823;
  case 2822:
    return 2823;
  case 2823:
    if (pd.flagYue & YUE3 || pd.flagRuria & RURIA3) {
      return 2847;
    } else {
      return 2824;
    }
  case 2824:
    if (pd.flagKurumi & KURUMI3) {
      return 2825;
    } else {
      return 2826;
    }
  case 2825:
    return 2827;
  case 2826:
    return 2827;
  case 2827:
    *isAlt = true;
    return 3264;
  case 2828:
    if (pd.flagHinata & HINATA1 || pd.flagHimiko & HIMIKO2 || pd.flagMao & MAO1) {
      return 2832;
    } else {
      return 2829;
    }
  case 2829:
    if (pd.flagKurumi & KURUMI3) {
      return 2830;
    } else {
      return 2831;
    }
  case 2830:
    return 2852;
  case 2831:
    return 2853;
  case 2832:
    *isAlt = true;
    return 3262;
  case 2833:
    *isAlt = true;
    if (pd.flagHimiko & HIMIKO1) {
      return 2835;
    } else {
      return 2834;
    }
  case 2834:
    switch (alternative.result) {
    case 0: /* Ҥʤˤޤ */
      return 2840;
    case 1: /* ᤰߤϡĤʤߤ */
      if (pd.flagHinata & HINATA3) {
	pd.flagHinata |= HINATA4;
      }
      return 2841;
    }
  case 2835:
    switch (alternative.result) {
    case 0: /* Ҥʤˤޤ */
      return 2840;
    case 1: /* ᤰߤϡĤʤߤ */
      if (pd.flagHinata & HINATA3) {
	pd.flagHinata |= HINATA4;
      }
      return 2841;
    case 2: /* Ҥʤ򲡤ݤ */
      if (pd.flagHimiko & HIMIKO2) {
	pd.flagHimiko |= HIMIKO3;
      }
      return 2842;
    }
    /* block 0x11 */
  case 2840:
    return 2844;
  case 2841:
    return 2846;
  case 2842:
    if (pd.flagHimiko & HIMIKO3) {
      return 2853;
    } else {
      return 2854;
    }
  case 2843:
    return 2844;
  case 2844:
    return 2846;
  case 2845:
    return 2846;
  case 2846:
    if (pd.flagKurumi & KURUMI2) {
      return 2852;
    } else {
      return 2853;
    }
  case 2847:
    *isAlt = true;
    return 3264;
  case 2848:
    if (pd.flagYue & YUE3 || pd.flagRuria & RURIA3) {
      return 3132;
    } else {
      return 2854;
    }
  case 2852:
    if (pd.flagSarasa & SARASA3) {
      return 2861;
    } else {
      return 2862;
    }
  case 2853:
    return 3125;
  case 2854:
    return 3125;
    /* block 0x12 */
  case 2861:
    return 2863;
  case 2862:
    return 2863;
  case 2863:
    if (pd.flagSarasa & SARASA3) {
      return 2867;
    } else if (pd.flagKurumi & KURUMI4) {
      return 2864;
    } else if (pd.flagKurumi & KURUMI1) {
      return 2866;
    } else if (pd.flagKurumi & KURUMIb) {
      return 2866;
    } else {
      return 2865;
    }
  case 2864:
    return 2874;
  case 2865:
      if (pd.flagKurumi & KURUMI1) {
	return 2866;
      } else if (pd.flagHimiko & HIMIKO1) {
	return 2871;
      } else {
	return 2874;
      }
  case 2866:
    if (pd.flagHimiko & HIMIKO1) {
      return 2871;
    } else {
      return 2874;
    }
  case 2867:
    *isAlt = true;
    return 3265;
  case 2868: /* ending */
    return 3066;
  case 2869:
    return 3228;
  case 2871:
    *isAlt = true;
    return 3266;
  case 2872:
    return 2875;
  case 2874:
    return 2875;
  case 2875:
    if (pd.flagHimiko & HIMIKO4 || pd.flagMao & MAO3) {
      return 2880;
    } else {
      return 2881;
    }
    /* block 0x13 */
  case 2880:
    return 2882;
  case 2881:
    return 2882;
  case 2882:
    *isAlt = true;
    return 3267;
  case 2883:
    return 2884;
  case 2884:
    return 3128;
  case 2886:
    return 3128;
  case 2888:
    if (pd.flagHimiko & HIMIKO4 || pd.flagMao & MAO1) {
      return 2904;
    } else {
      return 3141;
    }
  case 2889:
    *isAlt = true;
    return 3268;
  case 2890:
    return 3141;
  case 2893:
    return 2895;
  case 2894:
    return 2895;
  case 2895:
    if (pd.flagRuria & RURIAa) {
      return 2901;
    } else {
      return 3135;
    }
    /* block 0x14 */
  case 2901:
    return 2902;
  case 2902:
    if (pd.flagYue & YUE3) {
      return 2932;
    } else if (pd.flagRuria & RURIA3) {
      return 2933;
    } else if (pd.flagHimiko & HIMIKO4 || pd.flagMao & MAO1) {
      return 2904;
    }
  case 2903:
    return 2904;
  case 2904:
    return 3141;
  case 2906:
    return 2908;
  case 2907:
    return 2908;
  case 2908:
    *isAlt = true;
    return 3271;
  case 2909:
    return 2911;
  case 2910:
    return 2911;
  case 2911:
    if (pd.flagHinata & HINATA1 || pd.flagHimiko & HIMIKO1) {
      return 2912;
    } else {
      return 2921;
    }
  case 2912:
    *isAlt = true;
    return 3272;
  case 2913:
    if (pd.flagHimiko & HIMIKO4 || pd.flagMao & MAO4) {
      return 2914;
    } else {
      return 2915;
    }
  case 2914:
    return 3144;
  case 2915:
    return 3144;
    /* block 0x15 */
  case 2920:
    return 3144;
  case 2921:
    return 3144;
  case 2923:
    *isAlt = true;
    return 3273;
  case 2925:
    if (pd.flagHinata & HINATA5) {
      return 3174;
    } else {
      return 2926;
    }
  case 2926:
    return 3146;
  case 2928:
    return 3146;
  case 2929:
    return 3146;
  case 2932:
    return 2934;
  case 2933:
    return 2934;
  case 2934:
    if (pd.flagYue & YUE3) {
      return 2935;
    } else if (pd.flagRuria & RURIA3) {
      return 2940;
    }
  case 2935:
    return 2941;
    /* block 0x16 */
  case 2940:
    return 2941;
  case 2941:
    *isAlt = true;
    return 3274;
  case 2942: /* ending */
    return 3066;
  case 2943:
    if (pd.flagRuria & RURIA3) {
      return 3149;
    } else {
      return 2945;
    }
  case 2945:
    *isAlt = true;
    return 3275;
  case 2947:
    return 2948;
  case 2948:
    if (pd.flagYue & YUE4) {
      return 2949;
    } else {
      return 2950;
    }
  case 2949:
    return 2951;
  case 2950:
    return 2951;
  case 2951:
    *isAlt = true;
    return 3280;
  case 2952:
    return 3154;
  case 2953:
    return 3154;
  case 2954:
    return 2955;
  case 2955:
    return 3169;
    /* block 0x17 */
  case 2961:
    return 3169;
  case 2963:
    return 2965;
  case 2964:
    *isAlt = true;
    return 3282;
  case 2965: /* ending */
    return 3066;
  case 2972:
    *isAlt = true;
    return 3283;
  case 2973:
    return 3188;
    /* block 0x18 */
  case 2980:
    if (pd.flagYue & YUE5) {
      return 2981;
    } else {
      return 2982;
    }
  case 2981:
    *isAlt = true;
    return 3284;
  case 2982:
    return 2984;
  case 2984: /* ending */
    return 3066;
  case 2985:
    *isAlt = true;
    return 3285;
  case 2987:
    return 2988;
  case 2988:
    if (pd.flagRuria & RURIA4) {
      return 2989;
    } else {
      return 2991;
    }
  case 2989:
    *isAlt = true;
    return 3286;
  case 2990:
    return 2992;
  case 2991:
    return 2994;
  case 2992:
    return 2993;
  case 2993:
    return 2995;
  case 2994:
    return 2995;
  case 2995:
    if (pd.flagRuria & RURIA5) {
      return 3000;
    } else {
      return 3001;
    }
    /* block 0x19 */
  case 3000:
    return 3002;
  case 3001:
    return 3002;
  case 3002:
    if (pd.flagRuria & RURIA5) {
      return 3194;
    } else {
      return 3003;
    }
  case 3003: /* ending */
    return 3066;
  case 3006:
    return 3007;
  case 3007:
    return 3201;
  case 3008:
    return 3172;
  case 3009: /* ending */
    return 3073;
  case 3011:
    return 3013;
  case 3013:
    if (pd.flagHimiko & HIMIKO7) {
      return 3210;
    } else {
      return 3014;
    }
  case 3014:
    return 3066;
    /* block 0x1a */
  case 3020:
    if (pd.flagYue & YUE3 || pd.flagRuria & RURIA3) {
      return 3021;
    } else if (pd.flagKurumi & KURUMIb) {
      return 3022;
    } else {
      return 3023;
    }
  case 3021:
    return 3214;
  case 3022:
    return 3214;
  case 3023:
    return 3214;
  case 3025:
    return 3027;
  case 3026:
    return 3027;
  case 3027:
    if ((pd.flagClear & FLAG_CLEAR_MEGUMI) &&
	pd.flagKurumi & KURUMI2) {
      return 3029;
    } else {
      return 3028;
    }
  case 3028:
    return 3032;
  case 3029:
    *isAlt = true;
    return 3253;
  case 3030:
    return 3032;
  case 3031:
    return 3032;
  case 3032:
    if (pd.flagSarasa & SARASA3) {
      return 2790;
    } else if (pd.flagYue & YUE3 || pd.flagRuria & RURIA3) {
      return 3105;
    } else {
      return 2792;
    }
  case 3033:
    *isAlt = true;
    return 3288;
  case 3034:
    return 3040;
  case 3035:
    return 3040;
    /* block 0x1b */
  case 3040:
    if (pd.flagMinako & MINAKO1) {
      return 3041;
    } else {
      return 3220;
    }
  case 3041:
    return 3222;
  case 3042:
    return 3111;
  case 3044:
    return 3046;
  case 3045:
    return 3046;
  case 3046:
    *isAlt = true;
    return 3289;
  case 3047:
    if (pd.flagMinako & MINAKO1) {
      return 3048;
    } else {
      return 3293;
    }
  case 3048: /* ending */
    return 3066;
  case 3051:
    *isAlt = true;
    return 3290;
  case 3052: /* ending */
    return 3066;
  case 3055:
    return 3234;
    /* block 0x1c */
  case 3062:
    return 3066;
  case 3063:
    *isAlt = true;
    return 3292;
  case 3064:
    return 3066;
  case 3065: /* ending */
    return 3075;
  case 3066: /* Хåɥ */
    pd.Pos = FindStartMenu();
    return 3080;
  case 3067: /* ᤰߥ */
    pd.flagClear |= FLAG_CLEAR_MEGUMI;
    if (pd.flagClear == FLAG_CLEAR_ALL) {
      return 3346; /* ޤ */
    }
    pd.Pos = FindStartMenu();
    return 3080;
  case 3068: /* ͥͥ */
    pd.flagClear |= FLAG_CLEAR_YUE;
    if (pd.flagClear == FLAG_CLEAR_ALL) {
      return 3346; /* ޤ */
    }
    pd.Pos = FindStartMenu();
    return 3080;
  case 3069: /* ҥ */
    pd.flagClear |= FLAG_CLEAR_HIMIKO;
    if (pd.flagClear == FLAG_CLEAR_ALL) {
      return 3346; /* ޤ */
    }
    pd.Pos = FindStartMenu();
    return 3080;
  case 3070: /* ߥ */
    pd.flagClear |= FLAG_CLEAR_KURUMI;
    if (pd.flagClear == FLAG_CLEAR_ALL) {
      return 3346; /* ޤ */
    }
    pd.Pos = FindStartMenu();
    return 3080;
  case 3071: /* ҥ */
    pd.flagClear |= FLAG_CLEAR_MINAKO;
    if (pd.flagClear == FLAG_CLEAR_ALL) {
      return 3346; /* ޤ */
    }
    pd.Pos = FindStartMenu();
    return 3080;
  case 3072: /* Ҥʤ */
    pd.flagClear |= FLAG_CLEAR_HINATA;
    if (pd.flagClear == FLAG_CLEAR_ALL) {
      return 3346; /* ޤ */
    }
    pd.Pos = FindStartMenu();
    return 3080;
  case 3073: /* 泌 */
    pd.flagClear |= FLAG_CLEAR_MAO;
    if (pd.flagClear == FLAG_CLEAR_ALL) {
      return 3346; /* ޤ */
    }
    pd.Pos = FindStartMenu();
    return 3080;
  case 3074: /* ꥢ */
    pd.flagClear |= FLAG_CLEAR_RURIA;
    if (pd.flagClear == FLAG_CLEAR_ALL) {
      return 3346; /* ޤ */
    }
    pd.Pos = FindStartMenu();
    return 3080;
  case 3075: /* ӥ */
    pd.flagClear |= FLAG_CLEAR_SARASA;
    if (pd.flagClear == FLAG_CLEAR_ALL) {
      return 3346; /* ޤ */
    }
    pd.Pos = FindStartMenu();
    return 3080;
    /* block 0x1d */
  case 3080: /* opening */
    *isAlt = true;
    InitGame();
    return 3081;
  case 3081:
    *isAlt = true;
    switch (alternative.result) {
    case 0: /* Ϥ */
      return 3082;
    case 1: /* ̩Τ */
      return 3083;
    case 2: /* Ͽ򥯥ꥢ */
      return 3084;
    case 3: /* 򽪤 */
      return 3081;
    }
  case 3082:
    switch (alternative.result) {
    case 0: /* ǽ餫 */
      return 2645;
    case 1: /* ³ */
      *isAlt = true;
      return 3081;
    case 2: /*  */
      *isAlt = true;
      return 3081;
    }
  case 3083:
    *isAlt = true;
    switch (alternative.result) {
    case 0:  /* ҡ̩Х */
      return 3081;
    case 1: /*  */
      return 3081;
    case 2: /*  */
      return 3081;
    }
  case 3084:
    *isAlt = true;
    switch (alternative.result) {
    case 0:  /* Ͽ򥯥ꥢ */
      pd.flagClear = 0;
      return 3081;
    case 1:  /*  */
      return 3081;
    }
  case 3085:
    return 3086;
  case 3086:
    if (pd.flagClear & FLAG_CLEAR_YUE) {
      *isAlt = true;
      return 2653;
    } else {
      if (pd.flagYue & YUE1 || pd.flagRuria & RURIA1) {
	pd.flagYue |= YUE2;
	pd.flagRuria |= RURIA2;
      }
      return 2654;
    }
  case 3087:
    return 3088;
  case 3088:
    return 2694;
  case 3089:
    return 3090;
  case 3090:
    return 2724;
  case 3091:
    return 3092;
  case 3092:
    return 2731;
  case 3093:
    return 3094;
  case 3094:
    return 2771;
  case 3095:
    return 3100;
    /* block 0x1e */
  case 3100:
    if (pd.flagKurumi & KURUMI1) {
      return 2780;
    } else {
      return 2781;
    }
  case 3101:
    return 3102;
  case 3102:
    *isAlt = true;
    return 3252;
  case 3103:
    return 3104;
  case 3104:
    return 3020;
  case 3105:
    return 3106;
  case 3106:
    return 3042;
  case 3111:
    if (pd.flagKurumi & KURUMI2) {
      return 3330;
    } else {
      return 3331;
    }
  case 3112:
    if (pd.flagYue & YUE3 || pd.flagRuria & RURIA3) {
      return 2807;
    } else {
      return 2801;
    }
  case 3113:
    return 3114;
  case 3114:
    *isAlt = true;
    return 3255;
  case 3115:
    return 3120;
    /* block 0x1f */
  case 3120:
    return 2820;
  case 3121:
    return 3122;
  case 3122:
    *isAlt = true;
    return 3263;
  case 3123:
    return 3124;
  case 3124:
    return 3132;
  case 3125:
    return 3126;
  case 3126:
    return 3127;
  case 3127:
    if (pd.flagSarasa & SARASA3) {
      return 2861;
    } else {
      return 2862;
    }
  case 3128:
    return 3129;
  case 3129:
    if (pd.flagKurumi & KURUMI4) {
      return 2889;
    } else {
      return 2888;
    }
  case 3130:
    return 3131;
  case 3131:
    return 3033;
  case 3132:
    return 3133;
  case 3133:
    return 3134;
  case 3134:
    *isAlt = true;
    return 3269;
  case 3135:
    return 3140;
    /* block 0x20 */
  case 3140:
    return 2902;
  case 3141:
    return 3142;
  case 3142:
    return 3143;
  case 3143:
    if (pd.flagHinata & HINATA1 || pd.flagMao & MAO1 || pd.flagHimiko & HIMIKO1) {
      return 2907;
    } else {
      *isAlt = true;
      return 3270;
    }
  case 3144:
    return 3145;
  case 3145:
    if (pd.flagHimiko & HIMIKOc) {
      return 2923;
    } else {
      return 2929;
    }
  case 3146:
    return 3147;
  case 3147:
    return 3148;
  case 3148:
    return 3160;
  case 3149:
    return 3150;
  case 3150:
    return 3151;
  case 3151:
    return 2985;
  case 3152:
    return 3153;
  case 3153:
    return 2948;
  case 3154:
    return 3155;
  case 3155:
    if (pd.flagYue & YUE4) {
      return 2972;
    } else {
      return 2973;
    }
    /* block 0x21 */
  case 3160:
    return 3161;
  case 3161:
    return 3341;
  case 3163:
    return 3333;
  case 3165:
    return 3166;
  case 3166:
    return 3167;
  case 3167:
    *isAlt = true;
    return 3281;
  case 3169:
    return 3170;
  case 3170:
    if (pd.flagMegumi & MEGUMI3) {
      return 2964;
    } else {
      return 2963;
    }
  case 3171:
    return 3008;
  case 3172:
    return 3173;
  case 3173: /* ending */
    return 3067;
  case 3174:
    return 3175;
  case 3175:
    return 3180;
    /* block 0x22 */
  case 3180:
    return 3181;
  case 3181:
    return 3182;
  case 3182:
    return 3183;
  case 3183:
    return 3184;
  case 3184: /* ending */
    return 3072;
  case 3185:
    return 3186;
  case 3186:
    return 3187;
  case 3187:
    return 2980;
  case 3188:
    return 3189;
  case 3189:
    return 2980;
  case 3190:
    return 3191;
  case 3191: /* ending */
    return 3068;
  case 3192:
    return 3193;
  case 3193:
    return 2988;
  case 3194:
    return 3195;
  case 3195:
    return 3200;
    /* block 0x23 */
  case 3200: /* ending */
    return 3074;
  case 3201:
    return 3202;
  case 3202:
    return 3009;
  case 3203:
    return 3204;
  case 3204:
    *isAlt = true;
    return 3287;
  case 3205:
    return 3206;
  case 3206:
    return 3207;
  case 3207:
    return 3208;
  case 3208:
    return 3209;
  case 3209:
    return 3013;
  case 3210:
    return 3211;
  case 3211:
    return 3212;
  case 3212:
    return 3213;
  case 3213: /* ending */
    return 3069;
  case 3214:
    return 3215;
  case 3215:
    if (pd.flagKurumi & KURUMIa) {
      return 3325;
    } else {
      return 3326;
    }
    /* block 0x24 */
  case 3220:
    return 3221;
  case 3221:
    return 3222;
  case 3222:
    return 3223;
  case 3223:
    if (pd.flagMinako & MINAKO1) {
      return 3045;
    } else {
      return 3044;
    }
  case 3224:
    return 3225;
  case 3225: /* ending */
    return 3070;
  case 3226:
    return 3227;
  case 3227: /* ending */
    return 3071;
  case 3228:
    return 3229;
  case 3229:
    return 3230;
  case 3230:
    return 3231;
  case 3231:
    *isAlt = true;
    return 3291;
  case 3232:
    return 3233;
  case 3233:
    return 3234;
  case 3234:
    return 3235;
  case 3235:
    if (pd.flagSarasa & SARASA4) {
      return 3063;
    } else {
      return 3062;
    }
    /* block 0x25 */
  case 3240:
    switch (alternative.result) {
    case 0: /* 虜ȤäƤ */
      return 2661;
    case 1: /* ŷܤ */
      return 2662;
    }
  case 3241:
    switch (alternative.result) {
    case 0: /* εä */
      return 2670;
    case 1: /* ФäƤ */
      return 2671;
    case 2: /* ƤФä */
      return 2672;
    }
  case 3242:
    switch (alternative.result) {
    case 0: /* ;פʥȥ쥹Ϳʤ */
      return 2680;
    case 1: /* ͤ䤤ͤ */
      if (pd.flagSarasa & SARASA1) {
	pd.flagSarasa |= SARASA2;
      }
      return 2681;
    }
  case 3243:
    switch (alternative.result) {
    case 0: /* ޤǲظ */
      if (pd.flagSarasa & SARASA2) {
	pd.flagSarasa |= SARASA3;
      }
      return 3087;
    case 1: /* ޤ */
      return 2694;
    }
  case 3244:
    switch (alternative.result) {
    case 0: /* ǯζ̹ˤعԤ */
      return 2709;
    case 1: /* ̲Τݷ򼼤عԤ */
      pd.flagKurumi |= KURUMI1;
      pd.flagKurumi |= KURUMIa;
      return 2715;
    }
  case 3245:
    switch (alternative.result) {
    case 0: /* ǯζ̹ˤعԤ */
      return 2709;
    case 1: /* ̲Τݷ򼼤عԤ */
      pd.flagKurumi |= KURUMI1;
      pd.flagKurumi |= KURUMIa;
      return 2715;
    case 2: /* ҤιʲعԤ */
      pd.flagHimiko |= HIMIKOc;
      return 2720;
    }
  case 3246:
    switch (alternative.result) {
    case 0: /* ͤλ೼äƤ */
      pd.flagRuria |= RURIA3;
      return 2712;
    case 1: /* ͭΥǡȥݥå */
      pd.flagYue |= YUE3;
      return 2713;
    }
  case 3247:
    switch (alternative.result) {
    case 0: /* Τޤ */
      return 3089;
    case 1: /* ٤ */
      if (pd.flagHinata & HINATA1) {
	pd.flagHimiko |= HIMIKO1;
	pd.flagMao |= MAO1;
      }
      return 2722;
    }
  case 3248:
    return 2725;
  case 3250:
    switch (alternative.result) {
    case 0: /* ۤäƺǸޤʹ */
      return 2729;
    case 1: /* ʹʤ */
      if (pd.flagHimiko & HIMIKO1) {
	pd.flagHimiko |= HIMIKO2;
      }
      pd.flagHimiko |= HIMIKOa;
      return 3091;
    }
  case 3251:
    switch (alternative.result) {
    case 0: /* ҤߤǤʤ */
      return 2740;
    case 1: /* ̵Ǥ˰櫓ˤϤʤ */
      return 2743;
    }
  case 3252:
      switch (alternative.result) {
      case 0: /* ᤰߤ« */
	return 2787;
      case 1: /* ʹƤ */
	return 2788;
      }
  case 3253:
      switch (alternative.result) {
      case 0: /* ڤ򸫤˹Ԥ */
	if(pd.flagKurumi & KURUMI2) {
	  pd.flagKurumi |= KURUMI3;
	}
	return 3030;
      case 1: /* ᤰߤԤäƤΤǤ */
	return 3031;
      }
  case 3254:
    switch (alternative.result) {
    case 0: /* ˻ųݤ */
      return 2793;
    case 1: /* ľ˵ */
      return 2794;
    }
  case 3255:
    switch (alternative.result) {
    case 0: /* ҤϢ */
      return 3115;
    case 1: /* ٻϢ */
      return 2815;
    }
    /* block 0x26 */
  case 3260:
    switch (alternative.result) {
    case 0: /* ҤϢ */
      return 2811;
    case 1: /* ٻϢ */
      return 2815;
    }
  case 3262:
    switch (alternative.result) {
    case 0: /* 괺Ҥʤ򤫤餫 */
      return 3121;
    case 1: /* Ҥʤ̵뤷ƲĤƤΩʹ */
      return 2845;
    }
  case 3263:
    switch (alternative.result) {
    case 0: /* Ҥʤ˥ */
      if (pd.flagHinata & HINATA1) {
	pd.flagHinata |= HINATA3;
      } else if (pd.flagHimiko & HIMIKO2) {
	pd.flagHimiko |= HIMIKO3;
      }
      return 2833;
    case 1: /* ҤʤΥ */
      return 2843;
    }
  case 3264:
    switch (alternative.result) {
    case 0: /* ļͻҤ򸫤˹Ԥ */
      if (pd.flagKurumi & KURUMI3) {
	pd.flagKurumi |= KURUMI4;
      }
      if (pd.flagYue & YUE3 || pd.flagRuria & RURIA3) {
	return 3123;
      } else if (pd.flagKurumi & KURUMI3) {
	return 2828;
      } else {
	return 2828;
      }
    case 1: /* ȤʤԤäƤ */
      pd.flagHinata |= HINATAa;
      return 2848;
    }
  case 3265:
    switch (alternative.result) {
    case 0: /* ӤȰ˹Ԥ */
      return 2868;
    case 1: /* ˹Ԥ櫓ˤϤʤ */
      return 2869;
    }
  case 3266:
    switch (alternative.result) {
    case 0: /* ˿λä */
      if (pd.flagHimiko & HIMIKO3) {
	pd.flagHimiko |= HIMIKO4;
      }
      if (pd.flagMao & MAO1) {
	pd.flagMao |= MAO3;
      }
      return 2872;
    case 1: /* ޤо夵˲񤤤˹Ԥ*/
      return 2875;
    }
  case 3267:
    switch (alternative.result) {
    case 0: /* ˤᤰߤϤޤ±عԤ */
      pd.flagMegumi |= MEGUMI2;
      return 2883;
    case 1: /* ˸ذ⤹ */
      return 2886;
    }
  case 3268:
    switch (alternative.result) {
    case 0: /* ݤΤǡäƤ */
      return 2890;
    case 1: /* ͻҤ򸫤˸äƤ */
      if (pd.flagKurumi & KURUMI3) {
	pd.flagKurumi |= KURUMI5;
      }
      return 3130;
    }
  case 3269:
    switch (alternative.result) {
    case 0: /* ᤰߤޤ˹Ԥ */
      return 2893;
    case 1: /* ꥢޤ˹Ԥ */
      pd.flagRuria |= RURIAa;
      return 2894;
    }
  case 3270:
    switch (alternative.result) {
    case 0: /* ¸ */
      return 2906;
    case 1: /* ڡϻߤƤ */
      return 2907;
    }
  case 3271:
    switch (alternative.result) {
    case 0: /* ޤʤʤ */
      return 2909;
    case 1: /* ˲񤤤褿˷ޤäƤʡ */
      return 2910;
    }
  case 3272:
    switch (alternative.result) {
    case 0: /* ԿۤޤƤ롪 */
      if (pd.flagHimiko & HIMIKO5) {
	pd.flagHimiko |= HIMIKO6;
      }
      if (pd.flagMao & MAO3) {
	pd.flagMao |= MAO4;
      }
      if (pd.flagHinata & HINATA4) {
	pd.flagHinata |= HINATA5;
      }
      return 2913;
    case 1: /* ѤۤˤϴؤʤΤ֤ */
      return 2920;
    }
  case 3273:
    switch (alternative.result) {
    case 0: /* 狼ä˹Ԥ */
      return 2926;
    case 1: /* ˡäȥȥˤ */
      return 2925;
    case 2: /* ޽񼼤֤ܤäΤ˺Ƥ */
      if (pd.flagHimiko & HIMIKO4) {
	pd.flagHimiko |= HIMIKO5;
	return 3203;
      } else if (pd.flagMao & MAO4) {
	return 3006;
      } else {
	return 2928;
      }
    }
  case 3274:
    switch (alternative.result) {
    case 0: /* ߵ޼֤Ƥ */
      return 2942;
    case 1: /* ҤϢ */
      return 2943;
    }
  case 3275:
    switch (alternative.result) {
    case 0: /* ͥͤߤ */
      pd.flagYue |= YUE4;
      return 3152;
    case 1: /* ! ϤǤʤ! */
      return 2947;
    }
    /* block 0x27 */
  case 3280:
    switch (alternative.result) {
    case 0: /* ں */
      return 2952;
    case 1: /* ŰǺغ */
      return 2953;
    }
  case 3281:
    switch (alternative.result) {
    case 0: /* ᤰߤ */
      if (pd.flagMegumi & MEGUMI2) {
	pd.flagMegumi |= MEGUMI3;
      }
      return 2954;
    case 1: /* ñ夵 */
      return 2961;
    }
  case 3282:
    switch (alternative.result) {
    case 0: /* ̼Ͽ */
      return 2965;
    case 1: /* 󤸤ʤ */
      return 3171;
    }
  case 3283:
    switch (alternative.result) {
    case 0: /* ͥͤδ¤򤹤 */
      pd.flagYue |= YUE5;
      return 3185;
    case 1: /* Ҥμ򤹤 */
      return 3188;
    }
  case 3284:
    switch (alternative.result) {
    case 0: /* ʪ˿ΤϤ */
      return 3190;
    case 1: /* Ϥʤ */
      return 2984;
    }
  case 3285:
    switch (alternative.result) {
    case 0: /* ꥢܤˤͳʤ */
      pd.flagRuria |= RURIA4;
      return 3192;
    case 1: /* ͥͤϤɤ */
      return 2987;
    }
  case 3286:
    switch (alternative.result) {
    case 0: /* ҤΤ餻 */
      pd.flagRuria |= RURIA5;
      return 2990;
    case 1: /* ꥢȿڤعԤ */
      return 2991;
    }
  case 3287:
    switch (alternative.result) {
    case 0: /*  ޤä */
      return 3011;
    case 1: /* ŰǺ˴ */
      pd.flagHimiko |= HIMIKO7;
      return 3205;
    }
  case 3288:
    switch (alternative.result) {
    case 0: /* ζκڤ */
      if(pd.flagKurumi & KURUMI5) {
	pd.flagKurumi |= KURUMI6;
      }
      return 3034;
    case 1: /*  */
      pd.flagMinako |= MINAKO1;
      return 3035;
    }
  case 3289:
    switch (alternative.result) {
    case 0: /* о夵ޤԤȤ */
      return 3047;
    case 1: /* عƧ߹ */
      if (pd.flagMinako & MINAKO1) {
	return 3051;
      } else {
	return 3224;
      }
    }
  case 3290:
    switch (alternative.result) {
    case 0: /*  */
      return 3052;
    case 1: /*  */
      return 3226;
    }
  case 3291:
    switch (alternative.result) {
    case 0: /*  */
      return 3055;
    case 1: /* Ф˷ */
      pd.flagSarasa |= SARASA4;
      return 3232;
    }
  case 3292:
    switch (alternative.result) {
    case 0: /* Ҥ˹ӤϤ */
      return 3064;
    case 1: /* ӤȰ˹Ԥ */
      return 3065;
    }
  case 3293: /* ending */
    return 3066;
    /* block 0x29 */
  case 3325:
    return 3327;
  case 3326:
    return 3327;
  case 3327:
    if (pd.flagKurumi & KURUMI2) {
      return 3025;
    } else {
      return 3026;
    }
  case 3330:
    return 3332;
  case 3331:
    return 3332;
  case 3332:
    return 3112;
  case 3333:
    if (pd.flagMegumi & MEGUMI2) {
      return 3334;
    } else {
      return 3340;
    }
  case 3334:
    return 3335;
  case 3335:
    return 3165;
    /* block 0x2a */
  case 3340:
    return 3165;
  case 3341:
    return 3342;
  case 3342:
    return 3344;
  case 3343:
    return 3344;
  case 3344:
    return 3163;
  case 3345:
    return 3248;
  case 3346:
    *isAlt = true;
    return 3347;
  case 3347:
    switch (alternative.result) {
    case 0: /* ե¸ */
      break;
    case 1: /* ե¸ʤ */
      /* do nothing */
      break;
    }
  default:
    FrmAlert(ScrWarningAlert);
    return 2645;
  }
  
  return 0;
}
