/*
 * Hanashizume no Matsuri for Palm
 * Copyright (c) 2001 TF <tf@denpa.org>
 * All rights reserverd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: Text.c,v 1.8 2001/03/06 14:47:43 tf Exp $
 */
#include <PalmOS.h>
#include <PalmCompatibility.h>

#include "HanaRsc.h"
#include "Hana.h"

static void NewLine(void);

void
TextLoad(UInt32 id)
{
  VoidHand h;
  UInt16 idx, res;
  Err err;

  if (id == 0) {
    return;
  }

  err = DmFindRecordByID(dbR, id, &idx);
  if (err) {
    FrmAlert(ScrWarningAlert);
    return;
  }
  h = DmQueryRecord(dbR, idx);
  if (h == NULL) {
    FrmAlert(ScrWarningAlert);
    return;
  }
#if 0
    {
      UChar tmp[256];
      sprintf(tmp, "[<%d><%d><%d>]", (int)id, (int)idx, (int)pdbTable.len[idx - 1]);
      WinDrawChars(tmp, StrLen(tmp), 0, 0);
    }
#endif
  if (pdbTable.len[idx - 1] > MAX_TEXTSIZE) {
    /* error */
    FrmAlert(ScrWarningAlert);
    return;
  }
  
  MemMove(&dbimg, MemHandleLock(h), pdbTable.len[idx - 1]);
  MemHandleUnlock(h);
  res = DecodeLZ77(text, dbimg, pdbTable.len[idx - 1]);
  text[res] = '\0';
}

void
PutString(UChar *string, WinHandle w, Boolean controlNL)
{
  UChar buf[3], *p, *q;
  Int i = 0;

  if (pageBreak) {
    NewPage();
    pageBreak = false;
  }

  if (CurX == 0 && CurY ==0) {
    pd.topPos = topPos;
    pd.topBlock = pd.CurBlock;
    pd.topState = pd.state;
  }

  while (string[i] != '\0') {
    if (controlNL && CurX > 159) {
      NewLine();
      if (string[i] == 0x0a) { /* don't break line twice */
	i++;
	continue;
      }
    } else if (controlNL && 
	       CurX > 152 &&
	       !(string[i] == 0x81 && string[i + 1] == 0x41) && /*  */
	       !(string[i] == 0x81 && string[i + 1] == 0x42) && /*  */
	       !(string[i] == 0x81 && string[i + 1] == 0x48) && /*  */
	       !(string[i] == 0x81 && string[i + 1] == 0x49) && /*  */
	       !(string[i] == 0x81 && string[i + 1] == 0x6a) && /*  */
	       !(string[i] == 0x81 && string[i + 1] == 0x76) && /*  */
	       !(string[i] == 0x81 && string[i + 1] == 0x78) && /*  */
	       !(string[i] == 0x82 && string[i + 1] == 0xc1) && /*  */
	       !(string[i] == 0x82 && string[i + 1] == 0xe1) && /*  */
	       !(string[i] == 0x82 && string[i + 1] == 0xe3) && /*  */
	       !(string[i] == 0x82 && string[i + 1] == 0xe5) && /*  */
	       !(string[i] == 0x82 && string[i + 1] == 0x9f) && /*  */
	       !(string[i] == 0x82 && string[i + 1] == 0xa1) && /*  */
	       !(string[i] == 0x82 && string[i + 1] == 0xa3) && /*  */
	       !(string[i] == 0x82 && string[i + 1] == 0xa5) && /*  */
	       !(string[i] == 0x82 && string[i + 1] == 0xa7) && /*  */
	       !(string[i] == 0x83 && string[i + 1] == 0x83) && /*  */
	       !(string[i] == 0x83 && string[i + 1] == 0x85) && /*  */
	       !(string[i] == 0x83 && string[i + 1] == 0x87) && /*  */
	       !(string[i] == 0x83 && string[i + 1] == 0x62) && /*  */
	       !(string[i] == 0x83 && string[i + 1] == 0x40) && /*  */
	       !(string[i] == 0x83 && string[i + 1] == 0x42) && /*  */
	       !(string[i] == 0x83 && string[i + 1] == 0x44) && /*  */
	       !(string[i] == 0x83 && string[i + 1] == 0x46) && /*  */
	       !(string[i] == 0x83 && string[i + 1] == 0x48) ) {/*  */
      NewLine();
      if (string[i] == 0x0a) { /* don't break line twice */
	i++;
	continue;
      }
    } else if (controlNL && CurX > 143) {
      if (string[i] == 0x81 && string[i + 1] == 0x69) { /*  */
	NewLine();
      }
    }
    if (CurY > 159) {
      pageBreak = true;
      mode &= ~MODE_PAGESKIP;
      break;
    }
    
    if (string[i] == 0x0a) { /* newline */
      i++;
      NewLine();
    } else if (string[i] < 0x80) {
      skipTap = false;
      buf[0] = string[i++];
      buf[1] = '\0';
      ImageString(offscn.winTXT, buf, CurX, CurY);
      CurX += WIDTH_CH;
    } else {
      skipTap = false;
      buf[0] = string[i++];
      buf[1] = string[i++];
      buf[2] = '\0';
      ImageString(offscn.winTXT, buf, CurX, CurY);
      CurX += WIDTH_WC;
    }
  }
  
  if (pageBreak) {
    pd.topPos = topPos + i;
    pd.topBlock = pd.CurBlock;
    pd.topState = pd.state;
	
    p = string + i;
    q = string;
    while (*p != '\0') {
      *q++ = *p++;
    }
    *q = '\0';
    
    if (!skipTap) {
      CommandPause();
    }
  } else {
    string[0] = '\0';
  }
}

static void
NewLine()
{
  CurX = 0;
  CurY += HEIGHT_CH;
}

void
NewPage(void)
{
  CurX = CurY = 0;
  
  ClearTextWindow();
}

void
SetAlternative(UChar *string, WinHandle w)
{
  mode &= ~MODE_PAGESKIP;

  if (pd.CurBlock >= 3081 && pd.CurBlock <= 3084) {
    NewPage();
    CurY = 70;
    pd.topPos = topPos;
    pd.topBlock = pd.CurBlock;
    pd.topState = pd.state;
    pageBreak = false;
  } else {
    if (CurY > 159 - HEIGHT_CH * 4) {
      NewPage();
    }
    if (CurX > 0) {
      NewLine();
    }
    NewLine();
  }

  alternative.first = CurY;
  PutString(string, w, false);
  alternative.last = CurY;
    
  if (alternative.first > alternative.last) {
    /* when alternatives are at the new page */
    alternative.first = 0;
  }
  alternative.selected = false;
  alternative.result = -1;
  return;
}

Int
FindStartMenu(void)
{
  Int i = 0;

  TextLoad(3080);
  while (1) {
    if (text[i] != 'a') {
      i++;
      continue;
    }
    if (!StrNCompare(text + i, "ahnti05", 7)) {
      break;
    }
    i++;
  }
  return i;
}
