/*
 * Hanashizume no Matsuri for Palm
 * Copyright (c) 2001 TF <tf@denpa.org>
 * All rights reserverd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: cutter.c,v 1.6 2001/02/23 15:34:45 tf Exp $
 */
#include <stdio.h>
#include <stdlib.h>
#include "resinfo.h"
#include "cutter.h"

#define COMPRESS 1

#define TEXT_MAXLEN 8000

static int convert(FILE *, RESINFO *, const char *);
static int get_script(unsigned char *, int, unsigned char *);
static void put_compress_script(const char *, unsigned char *, int);
static int rearrange_script(unsigned char *, int, int, int);

const char *text_name[] = {
  "0x7", 
  "0x8", 
  "0x9", 
  "0xa", 
  "0xb", 
  "0xc", 
  "0xd", 
  "0xe", 
  "0xf", 
  "0x10", 
  "0x11", 
  "0x12", 
  "0x13", 
  "0x14", 
  "0x15", 
  "0x16", 
  "0x17", 
  "0x18", 
  "0x19", 
  "0x1a", 
  "0x1b", 
  "0x1c", 
  "0x1d", 
  "0x1e", 
  "0x1f", 
  "0x20", 
  "0x21", 
  "0x22", 
  "0x23", 
  "0x24", 
  "0x25", 
  "0x26", 
  "0x27", 
  "0x28", 
  "0x29", 
  "0x2a", 
};

#define TXT_NUM 36

int
main(int argc, char *argv[])
{
  FILE *fp;
  RESINFO *res_list;
  int i;

  res_list = create_rlist();
  if (argc != 3) {
    fprintf(stderr, "usage: cutter hanamsg.dll fontfile\n");
    exit(1);
  }
  if ((fp = fopen(argv[1], "rb")) == NULL) {
    fprintf(stderr, "Can't open %s.\n", argv[1]);
    exit(1);
  }
  scan_dll(fp, res_list);

  make_u2e_table(argv[2]);

  for (i = 0; i < TXT_NUM; i++) {
    convert(fp, res_list, text_name[i]);
  }
  
  return 0;
}

static int
convert(FILE *fp, RESINFO *r, const char *id)
{
  int i, b1, b2, size, buflen;
  unsigned char *block;
  char fname[32];
  char txtbuf[TEXT_MAXLEN];

  r = search_rlist(r, id);
  if (r == NULL) {
    fprintf(stderr, "Can't find the message \"%s\".\n", id);
    exit(1);
  }
  fseek(fp, r->offset, SEEK_SET);

  for (i = 0; i < 16; i++) {
    b1 = fgetc(fp);
    b2 = fgetc(fp);
    size = (b1 | (b2 << 8)) * 2;
    if (size) {
      sprintf(fname, "%s_%02d.txt", id, i);
      fprintf(stderr, "%s (size = %d)\n", fname, size);
      block = (unsigned char *)calloc(size, sizeof(unsigned char));
      fread(block, 1, size, fp);
      buflen = get_script(block, size, txtbuf);
      buflen = rearrange_script(txtbuf, buflen, strtol(id, NULL, 16), i);
      put_compress_script(fname, txtbuf, buflen);
      free(block);
    }
  }

  return 0;
}

static int
get_script(unsigned char *buf, int size, unsigned char *dest)
{
  int len = 0, i;

  for (i = 0; i < size; i += 2) {
    len += put_sjis(buf[i], buf[i + 1], dest + len);
  }
  *(dest + len) = '\0';
  len++;
  
  return len;
}

static int
rearrange_script(unsigned char *buf, int size, int id, int block)
{
  int i = 0, j = 0;
  int prev = 0, flip = 0;
  
  if (size < 100) { /* alternative or short script */
    return size;
  }

  while (i < size) {
    if (buf[i] < 0x80) { /* JIS X 0201 LH */
      if ((id == 0x1d && block == 0) ||
	  (id == 0x1c && block > 5)) { /* opening or ending */
	if (buf[i] == 0x20) { /* white space */
	  prev = buf[i];
	  i++;
	  continue;
	} else if (buf[i] == 0xa) { /* line break */
	  if (prev == 0xa && flip) {
	    prev = buf[i];
	    i++;
	    flip = 0;
	  } else {
	    prev = buf[i];
	    buf[j++] = buf[i++];
	    flip = 1;
	  }
	  continue;
	}
      }
      if (buf[i] == 0xa) {
	if (prev < 0 &&
	    !(buf[i - 2] == 0x81 && buf[i - 1] == 0x40) && /* garbage */
	    !(buf[i + 1] == 0x81 && buf[i + 2] == 0x40) && /* new paragraph */
	    buf[i + 1] != 'r' &&
	    buf[i + 1] != 'w') {
	  prev = buf[i];
	  i++;
	} else {
	  prev = buf[i];
	  buf[j++] = buf[i++];
	}
      } else {
	prev = buf[i];
	buf[j++] = buf[i++];
      }
    } else {
      prev = -1;
      if ((id == 0x1d && block == 0) ||
	  (id == 0x1c && block > 5)) { /* opening or ending */
	if (buf[i] == 0x81 && buf[i + 1] == 0x40) { /* double-byte white space */
	  i += 2;
	  continue;
	}
      }
      buf[j++] = buf[i++];
      buf[j++] = buf[i++];
    }
  }
  return j;
}

static void
put_compress_script(const char *filename, unsigned char *buf, int len)
{
  FILE *fp;
  unsigned char lz77buf[TEXT_MAXLEN];
  int lzlen;

#ifdef COMPRESS
  lzlen = lz77_compress(buf, len, lz77buf);

  fp = fopen(filename, "wb");
  fwrite(lz77buf, 1, lzlen, fp);
  fclose(fp);
#else
  fp = fopen(filename, "wb");
  fwrite(buf, 1, len, fp);
  fclose(fp);
#endif
}
