/*
 * Hanashizume no Matsuri for Palm
 * Copyright (c) 2001 TF <tf@denpa.org>
 * All rights reserverd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: unicode.c,v 1.4 2001/02/17 19:00:40 tf Exp $
 */
#include <stdio.h>
#include <unistd.h>
#ifndef __NO_UNIX
#include <sys/mman.h>
#endif
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>

#include "cutter.h"

static void euc2sjis(char *, char *);

static unsigned int u2e[0xffff];

#define BUFLEN 256

/* ESC code */
#define	SS2	((char)0x8e)

int
put_sjis(int b1, int b2, unsigned char *dst)
{
  int ucs;
  unsigned char euc[3], sjis[3];

  ucs = (b2 << 8) | b1;

  if (ucs < 0x100) {
    *dst = ucs;
    return 1;
  } else {
    euc[0] = u2e[ucs] >> 8;
    euc[1] = u2e[ucs] & 0xff;
    euc[2] = '\0';
    euc2sjis(euc, sjis);
    *dst = sjis[0];
    *(dst + 1) = sjis[1];
    return 2;
  }
}

static void
euc2sjis(char *euc, char *sjis)
{
  int c, c1, c2;
  unsigned char *e = (unsigned char *)euc;
  unsigned char *s = (unsigned char *)sjis;
     
  while (c = *e++) {
    if (c & 0x80) {
      if(c == SS2) {
	*s++ = *e++;
	continue;
      }
      c1 = ((c - 0xa1) >> 1) + 0x81;
      if (c1 >= 0xa0) {
	c1 += 0x40;
      }
      c2 = (*e++) - 0xa1;
      if (c & 1) {
	c2 += 0x40;
      } else {
	c2 += 0x9e;
      }
      if (c2 > 0x7e) {
	c2++;
      }
      *s++ = c1;
      *s++ = c2;
    } else {
      *s++ = c;
    }
  }
  *s = '\0';
}

#ifndef __NO_UNIX
int
make_u2e_table(const char *fontfile)
{
  int i, p;
  int fd;
  struct stat sb;
  unsigned char *addr;

  if ((fd = open(fontfile, 0, O_RDONLY)) < 0) {
    fprintf(stderr, "\n***************************************************\n");
    fprintf(stderr, " Can't open %s file.\n", fontfile);
    fprintf(stderr, " Make sure to do 'make install' after compilation.\n");
    fprintf(stderr, "***************************************************\n");
    exit(1);
  }
  
  if (fstat(fd, &sb) < 0 ||
      (addr = mmap(NULL, sb.st_size, PROT_READ, MAP_PRIVATE, fd, 0))
      == MAP_FAILED) {
    perror("mmap");
    exit(1);
  }
  close(fd);
  
  for (i = 0x391, p = 0; i < 0xffff; i++, p += 2) {
    u2e[i] = (addr[p] << 8) | addr[p + 1];
  }
  
  return 0;
}
#else
int
make_u2e_table(const char *fontfile)
{
  FILE *fp;
  int i;
  int u, l;

  fp = fopen(fontfile, "rb");
  for (i = 0x391; i < 0xffff; i++) {
    u = fgetc(fp);
    l = fgetc(fp);
    if (u == EOF || l == EOF) {
      break;
    }
    u2e[i] = (u << 8) | l;
  }
  fclose(fp);

  return 0;
}
#endif
